/*
 Vytvoří číslo dokladu v požadovaném formátu. Základní stráktura: [prefix][číslo][postfix]
 
 Vstupní parametry:
 - i_Prefix      - prefix výsledného formátu
 - i_CisAkt      - aktuální číslo, které bylo přiděleno řadou
 - i_Postfix     - postfix výsledného formátu
 - i_ZobrazNuly  - zda se má generované číslo doplnit zepředo nulami do délky dané i_CisDelka 
 - i_CisDelka    - délka čísla
 - i_Rok         - rok, pro který se doklad generuje

 Návratové hodnoty:
 - vygenerované číslo, příp. jeden z chybových stavů:
   * x   - Přetečení délky číselné řady 
   * xx  - Délka vygenerovaného čísla je delší než 20 
   * xxx - Maximální délka čísla nesmí být delší než 12 znaků (viz. manuál)

 Verze pro PostgreSQL:     Pavel Cvrček
 Původní verze pro Sybase: Josef Weinreb
*/

create or replace function dVytvorCisDokl 
(i_Prefix varchar(8), i_CisAkt integer, i_Postfix varchar(8), i_ZobrazNuly boolean, i_CisDelka integer, i_idUcetniObdobi integer) returns varchar(20) as $$
declare
  o_CisDokl varchar(20);
  loc_Prefix varchar(8);
  loc_Postfix varchar(8);
  loc_ZobrazNuly boolean;
  loc_cisAkt integer;
  loc_CisDelka integer;
  loc_CisDokl integer;
  cislo_str varchar(20);
  loc_Rok integer;
  str_Rok varchar(10);
begin
  /* Inicializace lokálních proměnných */
  loc_Prefix:= i_Prefix;
  loc_Postfix:= i_Postfix;
  loc_ZobrazNuly:= i_ZobrazNuly;
  loc_cisAkt:= i_cisAkt;
  loc_CisDelka:= i_CisDelka;
  
  /* Počátační kontrola vstupních proměnných */
  if (i_CisAkt < 0) then
    raise exception 'dVytvorCisDokl(): Aktuální číslo řady je neplatné. Je: %', i_CisAkt;
  end if;

  /* Pokud je prefix či postfix null, nahraď za prázdný řetězec */
  if loc_Prefix is null then
    loc_Prefix:= '';
  end if;
  if loc_Postfix is null then
    loc_Postfix:= '';
  end if;
  
  /* Získá se číslo pro nahrazení roku a případně se před něj doplní nula, pokud není 4 znaky dlouhý */
  select RokProRadu into loc_Rok from nucetobdobi where iducetobdobi=i_idUcetniObdobi;
  str_Rok:= lpad (cast (loc_Rok as text), 4, '0');

  /* Náhrada masky roku */
  loc_Prefix:= replace (loc_Prefix, '%YYYY%', str_Rok);
  loc_Prefix:= replace (loc_Prefix, '%YYY%', substring (str_Rok from 2));
  loc_Prefix:= replace (loc_Prefix, '%YY%', substring (str_Rok from 3));
  loc_Prefix:= replace (loc_Prefix, '%Y%', substring (str_Rok from 4));  

  loc_Postfix:= replace (loc_Postfix, '%YYYY%', str_Rok);
  loc_Postfix:= replace (loc_Postfix, '%YYY%', substring (str_Rok from 2));
  loc_Postfix:= replace (loc_Postfix, '%YY%', substring (str_Rok from 3));
  loc_Postfix:= replace (loc_Postfix, '%Y%', substring (str_Rok from 4)); 
  
  if loc_ZobrazNuly = false then
    o_CisDokl:= rtrim(loc_Prefix) || ltrim(cast (loc_CisAkt as text)) || rtrim(loc_Postfix);
  else
    o_CisDokl:= rtrim(loc_Prefix) || lpad(ltrim (cast (loc_CisAkt as text)), i_CisDelka, '0') || rtrim (loc_Postfix);
  end if;

  cislo_str:= ltrim(cast (loc_CisAkt as text));  

  /* Přetečení rozsahu číselné řady */
  if Length(ltrim(cislo_str)) > i_CisDelka then
    o_CisDokl:= 'x';
  end if;

  /* Délka čísla je delší než 20 */
  if Length(o_CisDokl) > 20 then
    o_CisDokl:= 'xx';
  end if;
  
  /* Maximální délka čísla nesmí být delší než 12 znaků (viz. manuál) */
  if Length (cislo_str)>12 then
    o_CisDokl:= 'xxx';
  end if;

  return o_CisDokl;
end;
$$ LANGUAGE plpgsql;