/*
 Funkce provede vrácení čísla dokladu z aplikace. Vrací inforamaci, zda se číslo povedlo vrátit či nikoliv.
   
 Vstupní parametry:
 - i_idRady       - id příslušné řady
 - i_idUcetObdobi - id účetního období
 - i_CisDokl      - číslo vraceného dokladu
  
 Autor verze pro PostgreSQL:     Pavel Cvrček
 Autor původní verze pro Sybase: Josef Weinreb
*/

create or replace function dVratCisDokl (in i_idRady integer,in i_idUcetObdobi integer,in i_CisDokl varchar(20)) returns boolean as $$
declare
  _CisDelka integer;
  _ZobrazNuly boolean;
  _Prefix varchar(8);
  _Postfix varchar(8);
  _CisAkt integer;
  _unique boolean;
  _modul varchar;
  _typPohybu varchar;
  _idRadyDiry integer;
  _idRadyRok integer;
  _CisPoc integer;
  _idRady integer;
  _idUcetObdobi integer;
  _idRadyDiryT integer;

  _CisDokl varchar(20);
begin
  /** Nejprve se zkontrolují vstupní parametry **/    
  
  /* Id dokladové řady nesmí být null */
  /* !!! Důležité !!! Podmínky na následujících řádcích nevyhazují výjimku kvůli
     importu z W6 na W10. */
  if (i_idRady is null) then
    return true;
    /*raise exception 'dVratCisDokl(): Vstupní číslo dokladové řady nesmí být prázdné!';*/
  end if;
  
  /* Kontrola existence id řady */
  select idRady into _idRady from dRady where idRady = i_idRady;
  if (_idRady is null) then
    return true;
    /*raise exception 'dVratCisDokl(): Pokus o vrácení čísla do dokladové řady, která neexistuje!';*/
  end if;
  
  if (i_idUcetObdobi is null) then
    raise exception 'dVratCisDokl(): Číslo vstupního účetního období je null!';
  end if;
  
  select idUcetObdobi into _idUcetObdobi from nUcetObdobi where idUcetObdobi=i_idUcetObdobi;
  if (_idUcetObdobi is null) then
    raise exception 'dVratCisDokl(): Číslo vstupního účetního období neodpovídá existujícímu účetnímu období!';
  end if;
  
  /** Konec striktní kontroly vstupncích údajů **/
  
  /* Zjistí se číslo, které je na vrcholu dokladové řady pro účetní období */
  select CisDelka,ZobrazNuly,CisAkt,CisPoc,Prefix,Postfix,idRadyRok into _CisDelka,_ZobrazNuly,_CisAkt,_CisPoc,_Prefix,_Postfix,
    _idRadyRok from dRadyRok where idRady = i_idRady and idUcetObdobi = i_idUcetObdobi for update;
  
  /* Zjistí se modul a typ pohybu, pro kterou řada je. */
  select modul into _modul from dRady where idRady = i_idRady;
  
  /* Vytvoř nové číslo podle vrcholu čís. řady pro porovnání */
  _CisDokl:= (select dVytvorCisDokl(_Prefix,_CisAkt,_Postfix,_ZobrazNuly,_CisDelka,i_idUcetObdobi));        
  
  /* Porovnám číslo z vrcholu řady s navráceným číslem. Pokud se jedná o poslední číslo řady, jednoduše snížím čítač,
     protože novější číslo nebylo přiřazeno. Pokud ne, vložím číslo do seznamu děr.*/
  if (rtrim(_CisDokl) = rtrim(i_CisDokl)) then
     update dRadyDiry set LastUpdate=now() where idRadyRok = _idRadyRok and cisdira = _CisDokl;
     
     /* Pokud se maže doklad, který je na vrcholu a není v díře */
     select idRadyDiry into _idRadyDiryT from dRadyDiry where cisdira = _CisDokl and idRadyRok = _idRadyRok and LastUpdate<=now();
     if (_idRadyDiryT is null and _CisAkt <> _CisPoc) then
       _CisAkt:= _CisAkt - 1;
     end if;
     
     loop
       _CisDokl:= (select dVytvorCisDokl(_Prefix,_CisAkt,_Postfix,_ZobrazNuly,_CisDelka,i_idUcetObdobi));
       select idRadyDiry into _idRadyDiry from dRadyDiry where cisdira = _CisDokl and idRadyRok = _idRadyRok and LastUpdate<=now();
       if (_idRadyDiry is not null) then
         delete from dRadyDiry where cisdira = _CisDokl and idRadyRok = _idRadyRok and LastUpdate<=now();
         
         /* Kontrola, aby nedošlo k přetečení počátku. */
         if (_CisAkt <> _CisPoc) then
           _CisAkt:= _CisAkt - 1;
         end if;
       else       
         exit;
       end if;
     end loop;
    
     update dRadyRok set cisAkt = _cisAkt where idRadyRok = _idRadyRok;
  else
    /* Nejsem na vrcholu rady - změním rezervaci na díru nebo díru založím, při opakovaném volání díru nevytvořím */
    select idRadyDiry into _idRadyDiry from dRadyDiry where idRadyRok = _idRadyRok and CisDira = i_CisDokl;
    if(_idRadyDiry is null) then
      insert into dRadyDiry( idRadyRok,CisDira,LastUpdate) values( _idRadyRok,i_CisDokl,now());
    else
      update dRadyDiry set LastUpdate = now() where idRadyDiry = _idRadyDiry;
    end if;
  end if;
  
  return true;
end;
$$ LANGUAGE plpgsql;