--------------------------------------------------------------------------------
-- Bug 46227 - Změna základní sazby DPH v Rumunsku - snížení z 24 % na 20 % (2016) a snížení na 19 % (2017) 
-- Bug 46232 - Oprava snížené sazby DPH u Kypru - 8 % opravit na 9 % a prohodit s druhou sníženou sazbou (5%) 
-- Bug 46233 - Další chyby v číselníku sazeb DPH - opravy k 1.1.2017
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION doplnSazbu(_statKod varchar, _szbDph numeric, _typSzbDphSufix varchar, _platiOd date, _platiDo date)
RETURNS void AS $$
DECLARE
    _uuid varchar;
    _idstatu integer;
    _typSzbDphK varchar;
    _idsazby integer;
BEGIN
    _uuid = _statKod || '_' || regexp_replace(to_char(_szbDph, '09D00'), '[^0-9]', '', 'g') || '_' || date_part('year', _platiOd) || case _typSzbDphSufix when 'Sniz2' then '_2' else '' end;
    _typSzbDphK = 'typSzbDph.dph' || _typSzbDphSufix;

    SELECT idstatu INTO _idstatu FROM astaty WHERE uuid like '%_' || _statKod;
    if _idstatu is null then
        return;
    end if;

    SELECT idsazbydph INTO _idsazby FROM usazbydph WHERE idstatu = _idstatu and szbDph = _szbDph and typSzbDphK = _typSzbDphK and platiOdData < coalesce(_platiDo, now()) ORDER BY platiOdData desc limit 1;
    if _idsazby is not null then
        UPDATE usazbydph SET uuid = _uuid WHERE idsazbydph = _idsazby AND coalesce(uuid, '') <> _uuid;
        if _platiDo is not null then
            UPDATE usazbydph SET platiDoData = _platiDo WHERE uuid = _uuid AND coalesce(platiDoData, '2000-01-01') <> _platiDo;
        end if;
    else
        INSERT INTO usazbydph (lastUpdate, idstatu, szbDph, typSzbDphK, platiOdData, platiDoData, uuid)
        VALUES (now(), _idstatu, _szbDph, _typSzbDphK, _platiOd, _platiDo, _uuid);
    end if;
END;
$$ LANGUAGE plpgsql;


--RO
SELECT doplnSazbu('RO', 24.00, 'Zakl', '2015-01-01', '2015-12-31');
SELECT doplnSazbu('RO', 20.00, 'Zakl', '2016-01-01', '2016-12-31');
SELECT doplnSazbu('RO', 19.00, 'Zakl', '2017-01-01', NULL);


--AT
--oprava puvodne nesmyslne uvedene 2. snizene sazby
UPDATE usazbydph SET uuid = 'AT_1000_2016_2', platiOdData = '2016-01-01' WHERE uuid = 'AT_1000_2015_2';
SELECT doplnSazbu('AT', 10.00, 'Sniz',  '2015-01-01', '2015-12-31');
SELECT doplnSazbu('AT', 13.00, 'Sniz',  '2016-01-01', NULL);
SELECT doplnSazbu('AT', 10.00, 'Sniz2', '2016-01-01', NULL);


--GR
SELECT doplnSazbu('GR', 23.00, 'Zakl',  '2015-01-01', '2016-05-31');
SELECT doplnSazbu('GR', 24.00, 'Zakl',  '2016-06-01', NULL);
SELECT doplnSazbu('GR',  6.50, 'Sniz2', '2015-01-01', '2015-07-19');
SELECT doplnSazbu('GR',  6.00, 'Sniz2', '2015-07-20', NULL);


--LU
--potrebujeme opravit uuid
SELECT doplnSazbu('LU', 17.00, 'Zakl', '2015-01-01', NULL);
SELECT doplnSazbu('LU',  8.00, 'Zakl', '2015-01-01', NULL);
SELECT doplnSazbu('LU',  3.00, 'Zakl', '2015-01-01', NULL);


--CY
CREATE OR REPLACE FUNCTION fixCY()
RETURNS void AS $$    
DECLARE
BEGIN    
    --oprava chybne uvedene sazby
    UPDATE usazbydph SET szbDph = 9.00, uuid = 'CY_0900_2015_2' WHERE uuid = 'CY_0800_2015_2' and szbDph <> 9.00;

    begin
        --obe snizene sazby zkusime smazat
        DELETE FROM usazbydph WHERE uuid in ('CY_0500_2015', 'CY_0900_2015_2');
    
        --aktualizace rucne vytvorenych / doplneni
        PERFORM doplnSazbu('CY', 9.00, 'Sniz',  '2015-01-01', NULL);
        PERFORM doplnSazbu('CY', 5.00, 'Sniz2', '2015-01-01', NULL);

    exception when foreign_key_violation then
        raise notice 'Sazby DPH Kypru nelze opravit, jsou použité.';
    end;
END;
$$ LANGUAGE plpgsql;

SELECT fixCY();
DROP FUNCTION fixCY();


--MT
CREATE OR REPLACE FUNCTION fixMT()
RETURNS void AS $$    
DECLARE
BEGIN    
    begin
        --chybne zarazenou sazbu zkusime smazat
        DELETE FROM usazbydph WHERE uuid = 'MT_0500_2015';

        --aktualizace rucne vytvorenych / doplneni
        PERFORM doplnSazbu('MT', 7.00, 'Sniz',  '2015-01-01', NULL);
        PERFORM doplnSazbu('MT', 5.00, 'Sniz2', '2015-01-01', NULL);

    exception when foreign_key_violation then
        raise notice 'Sazby DPH Malty nelze opravit, jsou použité.';
    end;
END;
$$ LANGUAGE plpgsql;

SELECT fixMT();
DROP FUNCTION fixMT();


--FR
CREATE OR REPLACE FUNCTION fixFR()
RETURNS void AS $$    
DECLARE
BEGIN    
    begin
        --obe snizene sazby zkusime smazat
        DELETE FROM usazbydph WHERE uuid in ('FR_0550_2015', 'FR_0210_2015_2');
    
        --aktualizace rucne vytvorenych / doplneni
        PERFORM doplnSazbu('FR', 10.00, 'Sniz',  '2015-01-01', NULL);
        PERFORM doplnSazbu('FR',  5.50, 'Sniz2', '2015-01-01', NULL);

    exception when foreign_key_violation then
        raise notice 'Sazby DPH Francie nelze opravit, jsou použité.';
    end;
END;
$$ LANGUAGE plpgsql;

SELECT fixFR();
DROP FUNCTION fixFR();


--LV (Lotyšsko)
CREATE OR REPLACE FUNCTION fixLV()
RETURNS void AS $$    
DECLARE
BEGIN    
    --oprava chybne uvedene sazby
    UPDATE usazbydph SET szbDph = 12.00, uuid = 'LV_1200_2015' WHERE uuid = 'LV_0900_2015' and szbDph <> 12.00;

    begin
        --chybnou sazbu zkusime smazat
        DELETE FROM usazbydph WHERE uuid = 'LV_0500_2015';

    exception when foreign_key_violation then
        raise notice 'Sazby DPH Lotyšska nelze opravit, jsou použité.';
    end;
END;
$$ LANGUAGE plpgsql;

SELECT fixLV();
DROP FUNCTION fixLV();


--LT (Litva)
--oprava chybne uvedene sazby
UPDATE usazbydph SET szbDph = 9.00, uuid = 'LT_0900_2015' WHERE uuid = 'LT_1200_2015' and szbDph <> 9.00;
SELECT doplnSazbu('LT', 5.00, 'Sniz2', '2015-01-01', NULL);


--smazani neexistujicich sazeb
CREATE OR REPLACE FUNCTION odstranNesmysly()
  RETURNS void AS $$
declare
  sazby varchar[] := '{ BG_0900_2015_2, DE_0700_2015_2, DK_0000_2015, DK_0000_2015_2, EE_0900_2015_2, GB_0500_2015_2, LT_1200_2015_2, MT_0500_2015_2, NL_0600_2015_2, SI_0950_2015_2 }';
BEGIN
    FOR i IN array_lower(sazby, 1) .. array_upper(sazby, 1) LOOP
        BEGIN
            DELETE FROM usazbydph WHERE uuid = sazby[i];
        EXCEPTION WHEN foreign_key_violation THEN
            RAISE NOTICE 'Sazbu DPH uuid = % nelze vymazat, používá se.', sazby[i];
        END;
    END LOOP;
END;
$$ LANGUAGE plpgsql;

SELECT odstranNesmysly();
DROP FUNCTION odstranNesmysly();


--doplneni nulovych sazeb
SELECT doplnSazbu('AT', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('BE', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('BG', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('DE', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('DK', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('EE', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('ES', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('FI', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('FR', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('GB', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('GR', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('HR', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('HU', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('IE', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('IT', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('LT', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('LU', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('LV', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('MT', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('NL', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('PL', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('PT', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('RO', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('SE', 0.00, 'Osv', '2015-01-01', NULL);
SELECT doplnSazbu('SI', 0.00, 'Osv', '2015-01-01', NULL);


DROP FUNCTION doplnSazbu(varchar, numeric, varchar, date, date);


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 1067, 't', %appVersion%, clock_timestamp());
