-----------------------------------------------------------------------
-- Bug 16008 – Zamykání dokladů a zamykání období
-----------------------------------------------------------------------

-- Zamykani dokladu

ALTER TABLE ddoklfak
  ADD COLUMN zamekk character varying(50) default 'zamek.otevreno';
ALTER TABLE ddoklint
  ADD COLUMN zamekk character varying(50) default 'zamek.otevreno';
ALTER TABLE ddoklsklad
  ADD COLUMN zamekk character varying(50) default 'zamek.otevreno';
ALTER TABLE ddoklobch
  ADD COLUMN zamekk character varying(50) default 'zamek.otevreno';

-- Převedení hodnoty pole zamek na zamekK (jen u těch zamčených)

UPDATE ddoklfak SET zamekk = 'zamek.zamceno' WHERE zamek = true;
UPDATE ddoklint SET zamekk = 'zamek.zamceno' WHERE zamek = true;
UPDATE ddoklsklad SET zamekk = 'zamek.zamceno' WHERE zamek = true;
UPDATE ddoklobch SET zamekk = 'zamek.zamceno' WHERE zamek = true;


-----------------------------------------------------------------------
-- Zamykani obdobi

CREATE SEQUENCE seq_wzamky
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_wzamky OWNER TO %firmaRole%;

CREATE TABLE wzamky (
    idzamku integer DEFAULT nextval('seq_wzamky'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,

    zamekk character varying(50),
    platioddata date NOT NULL,
    platidodata date NOT NULL,

    modulfav boolean DEFAULT false,
    modulfap boolean DEFAULT false,
    modulphl boolean DEFAULT false,
    modulzav boolean DEFAULT false,
    modulban boolean DEFAULT false,
    modulpok boolean DEFAULT false,
    modulint boolean DEFAULT false,
    modulskl boolean DEFAULT false,

    modulppp boolean DEFAULT false,
    modulppv boolean DEFAULT false,
    modulnap boolean DEFAULT false,
    modulnav boolean DEFAULT false,
    modulobp boolean DEFAULT false,
    modulobv boolean DEFAULT false,

    modulmaj boolean DEFAULT false,
    modullea boolean DEFAULT false,
    modulmzd boolean DEFAULT false,

    CONSTRAINT wzamky_pkey PRIMARY KEY (idzamku)
);

ALTER TABLE public.wzamky OWNER TO %firmaRole%;


-----------------------------------------------------------------------
-- K nastavení uživatele přibude zaškrtávátko, že se jedná o "účetní"

ALTER TABLE wuzivatel ADD COLUMN ucetni boolean DEFAULT false NOT NULL;


-- U uživatele s rolí Admin nebo Účetní je defaultně zapnuta funkce pro zamykání záznamů

UPDATE wuzivatel
SET ucetni = true
WHERE pristupk = 'typPrav.admin' OR pristupk = 'typPrav.ucetni';


-----------------------------------------------------------------------
-- Kontrola editace zamčených dokladů
/* Zakomentovano napr. kvuli moznosti uhradit zamcenou fakturu, prepocitani skladu, apod.
CREATE OR REPLACE FUNCTION dzkontrolujzamek() RETURNS trigger AS $$
begin
  -- test zda smi dojit ke zmene na dokladu
  if (OLD.zamekk = 'zamek.zamceno' and NEW.zamekk = 'zamek.zamceno') then
    raise exception 'dzkontrolujzamek(): Pokus o zmenu zamceneho zaznamu.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dzkontrolujzamek() OWNER TO %firmaRole%;

CREATE TRIGGER ddoklfakzkontrolujzamek
    BEFORE UPDATE ON ddoklfak
    FOR EACH ROW
    EXECUTE PROCEDURE dzkontrolujzamek();

CREATE TRIGGER ddoklintzkontrolujzamek
    BEFORE UPDATE ON ddoklint
    FOR EACH ROW
    EXECUTE PROCEDURE dzkontrolujzamek();

CREATE TRIGGER ddoklskladkontrolujzamek
    BEFORE UPDATE ON ddoklsklad
    FOR EACH ROW
    EXECUTE PROCEDURE dzkontrolujzamek();

CREATE TRIGGER ddoklobchzkontrolujzamek
    BEFORE UPDATE ON ddoklobch
    FOR EACH ROW
    EXECUTE PROCEDURE dzkontrolujzamek();


CREATE OR REPLACE FUNCTION dpolfakzkontrolujzamek() RETURNS trigger AS $$
begin
  -- test zda smi dojit ke zmene na dokladu
  if ((select zamekk from ddoklfak where iddoklfak = NEW.iddoklfak) = 'zamek.zamceno') then
    raise exception 'dpolfakzkontrolujzamek(): Pokus o zmenu zamceneho zaznamu.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolfakzkontrolujzamek() OWNER TO %firmaRole%;

CREATE TRIGGER dpolfakzkontrolujzamek
    BEFORE UPDATE ON dpolfak
    FOR EACH ROW
    EXECUTE PROCEDURE dpolfakzkontrolujzamek();


CREATE OR REPLACE FUNCTION dpolintzkontrolujzamek() RETURNS trigger AS $$
begin
  -- test zda smi dojit ke zmene na dokladu
  if ((select zamekk from ddoklint where iddoklint = NEW.iddoklint) = 'zamek.zamceno') then
    raise exception 'dpolintzkontrolujzamek(): Pokus o zmenu zamceneho zaznamu.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolintzkontrolujzamek() OWNER TO %firmaRole%;

CREATE TRIGGER dpolintzkontrolujzamek
    BEFORE UPDATE ON dpolint
    FOR EACH ROW
    EXECUTE PROCEDURE dpolintzkontrolujzamek();


CREATE OR REPLACE FUNCTION dpolskladzkontrolujzamek() RETURNS trigger AS $$
begin
  -- test zda smi dojit ke zmene na dokladu
  if ((select zamekk from ddoklsklad where iddoklsklad = NEW.iddoklsklad) = 'zamek.zamceno') then
    raise exception 'dpolskladzkontrolujzamek(): Pokus o zmenu zamceneho zaznamu.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolskladzkontrolujzamek() OWNER TO %firmaRole%;

CREATE TRIGGER dpolskladzkontrolujzamek
    BEFORE UPDATE ON dpolsklad
    FOR EACH ROW
    EXECUTE PROCEDURE dpolskladzkontrolujzamek();


CREATE OR REPLACE FUNCTION dpolobchzkontrolujzamek() RETURNS trigger AS $$
begin
  -- test zda smi dojit ke zmene na dokladu
  if ((select zamekk from ddoklobch where iddoklobch = NEW.iddoklobch) = 'zamek.zamceno') then
    raise exception 'dpolobchzkontrolujzamek(): Pokus o zmenu zamceneho zaznamu.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolobchzkontrolujzamek() OWNER TO %firmaRole%;

CREATE TRIGGER dpolobchzkontrolujzamek
    BEFORE UPDATE ON dpolobch
    FOR EACH ROW
    EXECUTE PROCEDURE dpolobchzkontrolujzamek();
*/


-----------------------------------------------------------------------
-- Odstranění kontroly editace zamčených dokladů z uložených procedur pro změnu účtu

CREATE OR REPLACE FUNCTION ddoklfakupdateucty() RETURNS trigger
    AS $$
declare nelzeZmena int;
        idDokl int;
begin
  nelzeZmena := 0;

  --pokud dojde ke zmene primarniho uctu
  if (compareId(NEW.idPrimUcet, OLD.idPrimUcet) = false) then
    -- uprava banka/pokladna vazbami uhrada a vazbaZDD, pouze pokud ucet nesouhlasi aby nedochazelo k zacykleni
    for idDokl in (select idDoklInt from dDoklInt where compareId(idProtiUcet, NEW.idPrimUcet) = false and idDoklInt in
        (select idTabB from dVazebTab where idTabA = NEW.idDoklFak and typVazbyK in ('typVazbyDokl.uhrada','typVazbyDokl.vazbaZDD')))
    loop
        execute dNastavUcetDokladu(1, idDokl, 1, NEW.idPrimUcet);
    end loop;

    -- uhradaKR, preceneni - u kurzoveho rozdilu a preceneni nevim na ktere strane je hledany ucet, testuji oba
    for idDokl in (select idDoklInt from dDoklInt where compareId(idProtiUcet, NEW.idPrimUcet) = false and compareId(idProtiUcet, OLD.idPrimUcet) = true and idDoklInt in
        (select idTabB from dVazebTab where idTabA = NEW.idDoklFak and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
    loop
        execute dNastavUcetDokladu(1, idDokl, 1, NEW.idPrimUcet);
    end loop;

    for idDokl in (select idDoklInt from dDoklInt where compareId(idPrimUcet,  NEW.idPrimUcet) = false and compareId(idPrimUcet,  OLD.idPrimUcet) = true and idDoklInt in
        (select idTabB from dVazebTab where idTabA = NEW.idDoklFak and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
    loop
        execute dNastavUcetDokladu(1, idDokl, 0, NEW.idPrimUcet);
    end loop;
  end if;

  --pokud dojde ke zmene protiuctu (doklad je zdd menime ucty odpoctu)
  if (compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then
    if (NEW.typPohybuK = 'typPohybu.prijem') then             -- kdyz je to faktura vydana
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idZklDalUcet, NEW.idProtiUcet) = false and kopZklDalUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idZklDalUcet = NEW.idProtiUcet where compareId(idZklDalUcet, NEW.idProtiUcet) = false and idPolFak in
            (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;
    end if;

    if (NEW.typPohybuK = 'typPohybu.vydej') then             -- kdyz je to faktura prijata
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idZklMdUcet, NEW.idProtiUcet) = false and kopZklMdUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idZklMdUcet = NEW.idProtiUcet where compareId(idZklMdUcet, NEW.idProtiUcet) = false and idPolFak in
            (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;

      -- vazba na sklad
      for idDokl in (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSkl' and idTabA = NEW.idDoklFak and
            compareId((select idProtiUcet from dDoklSklad where idDoklSklad = idTabB), NEW.idProtiUcet) = false)
      loop
        execute dNastavUcetDokladu(2, idDokl, 1, NEW.idProtiUcet);
      end loop;
    end if;
  end if;

  --pokud dojde ke zmene protiuctu zakladni sazby dph (doklad je zdd menime ucty odpoctu)
  if (compareId(NEW.idDphZaklUcet, OLD.idDphZaklUcet) = false and nelzeZmena = 0) then
    if (NEW.typPohybuK = 'typPohybu.prijem') then             -- kdyz je to faktura vydana
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idDphDalUcet, NEW.idDphZaklUcet) = false and typSzbDphK = 'typSzbDph.dphZakl' and kopDphDalUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idDphDalUcet = NEW.idDphZaklUcet where compareId(idDphDalUcet, NEW.idDphZaklUcet) = false and typSzbDphK = 'typSzbDph.dphZakl' and
            idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;
    end if;

    if (NEW.typPohybuK = 'typPohybu.vydej') then             -- kdyz je to faktura prijata
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idDphMdUcet, NEW.idDphZaklUcet) = false and typSzbDphK = 'typSzbDph.dphZakl' and kopDphMdUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idDphMdUcet = NEW.idDphZaklUcet where compareId(idDphMdUcet, NEW.idDphZaklUcet) = false and typSzbDphK = 'typSzbDph.dphZakl' and
            idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;
    end if;
  end if;

  --pokud dojde ke zmene protiuctu snizene sazby dph (doklad je zdd menime ucty odpoctu)
  if (compareId(NEW.idDphSnizUcet, OLD.idDphSnizUcet) = false and nelzeZmena = 0) then
    if (NEW.typPohybuK = 'typPohybu.prijem') then             -- kdyz je to faktura vydana
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idDphDalUcet, NEW.idDphSnizUcet) = false and typSzbDphK = 'typSzbDph.dphSniz' and kopDphDalUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idDphDalUcet = NEW.idDphSnizUcet where compareId(idDphDalUcet, NEW.idDphSnizUcet) = false and typSzbDphK = 'typSzbDph.dphSniz' and
            idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;
    end if;

    if (NEW.typPohybuK = 'typPohybu.vydej') then             -- kdyz je to faktura prijata
      -- zjistime zda neexistujou nake co nesmime zmenit
      select count(*) into nelzeZmena from dPolFak where compareId(idDphMdUcet, NEW.idDphSnizUcet) = false and typSzbDphK = 'typSzbDph.dphSniz' and kopDphMdUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      -- pokud smime zmenit vsechny polozky co potrebujeme menime
      if (nelzeZmena = 0) then
        update dPolFak set idDphMdUcet = NEW.idDphSnizUcet where compareId(idDphMdUcet, NEW.idDphSnizUcet) = false and typSzbDphK = 'typSzbDph.dphSniz' and
            idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idDoklFak and typVazbyK = 'typVazbyDokl.odpocetZDD');
      end if;
    end if;
  end if;

  if (nelzeZmena > 0) then
    raise exception 'dDoklFakUpdateUcty(): Nelze zm�nit ��et u nav�zan� polo�ky.';
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.ddoklfakupdateucty() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION ddoklintupdateucty() RETURNS trigger
    AS $$
declare idPoloz int;
        idDokl int;
begin
  --pokud dojde ke zmene protiuctu
  if (compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then

    -- uprava faktur vazbami uhrada a vazbaZDD, pouze pokud ucet nesouhlasi aby nedochazelo k zacykleni
    for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idProtiUcet) = false and idDoklFak in
        (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhrada','typVazbyDokl.vazbaZDD')))
    loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idProtiUcet);
    end loop;

    -- odpocet zalohy - polozka dokladu
    for idPoloz in (select idTabA from dVazebTab where typVazbyK = 'typVazbyDokl.odpocetZALOHY' and idTabB in
            (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK = 'typVazbyDokl.uhradaZALOHY')) loop
      if (NEW.typPohybuK = 'typPohybu.prijem') then
        update dPolFak set idZklDalUcet = NEW.idProtiUcet where idPolFak = idPoloz and compareId(idZklDalUcet, NEW.idProtiUcet) = false and compareId(idZklDalUcet, OLD.idProtiUcet) = true;
      else
        update dPolFak set idZklMdUcet  = NEW.idProtiUcet where idPolFak = idPoloz and compareId(idZklMdUcet,  NEW.idProtiUcet) = false and compareId(idZklMdUcet,  OLD.idProtiUcet) = true;
      end if;
    end loop;

    -- uhradaKR, preceneni
    for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idProtiUcet) = false and compareId(idPrimUcet, OLD.idProtiUcet) = true and idDoklFak in
        (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
    loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idProtiUcet);
    end loop;
  end if;

  if (compareId(NEW.idPrimUcet, OLD.idPrimUcet) = false) then
    -- uhradaKR, preceneni
    for idDokl in (select idDoklFak from dDoklFak where compareId(idPrimUcet, NEW.idPrimUcet) = false and compareId(idPrimUcet, OLD.idPrimUcet) = true and idDoklFak in
        (select idTabA from dVazebTab where idTabB = NEW.idDoklInt and typVazbyK in ('typVazbyDokl.uhradaKR','typVazbyDokl.preceneni','typVazbyDokl.uhradaZB')))
    loop
        execute dNastavUcetDokladu(0, idDokl, 0, NEW.idPrimUcet);
    end loop;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.ddoklintupdateucty() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION ddoklskladupdateucty() RETURNS trigger
    AS $$
declare idFak int;
begin
  if (NEW.typPohybuK = 'typPohybu.prijem' and compareId(NEW.idProtiUcet, OLD.idProtiUcet) = false) then
    for idFak in (select idTabA from dVazebTab where typVazbyK = 'typVazbyDokl.hlavaSkl' and idTabB = NEW.idDoklSklad and
            compareId((select idProtiUcet from dDoklFak where idDoklFak = idTabA), NEW.idProtiUcet) = false)
    loop
        execute dNastavUcetDokladu(0, idFak, 1, NEW.idProtiUcet);
    end loop;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.ddoklskladupdateucty() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION dpolfakupdateucty() RETURNS trigger
    AS $$
declare typPohyb character varying(50);
        nelzeZmena int;
        idDokl int;
begin
  if (NEW.typPolozkyK = 'typPolozky.katalog') then
      -- nacteni typu pohybu
      select typPohybuK into typPohyb from dDoklFak where idDoklFak = NEW.idDoklFak;

      -- prijata faktura
      if (typPohyb = 'typPohybu.vydej') then
        -- zjistime zda neexistujou nake co nesmime zmenit
        select count(*) into nelzeZmena from dPolSklad where compareId(idZklDalUcet, NEW.idZklMdUcet) = false and kopZklDalUcet = true and
            idPolSklad in (select idTabB from dVazebTab where idTabA = NEW.idPolFak and typVazbyK = 'typVazbyDokl.polozkaSkl');
        -- pokud smime zmenit vsechny polozky co potrebujeme menime
        if (nelzeZmena = 0) then
          update dPolSklad set idZklDalUcet = NEW.idZklMdUcet where compareId(idZklDalUcet, NEW.idZklMdUcet) = false and idPolSklad in
            (select idTabB from dVazebTab where idTabA = NEW.idPolFak and typVazbyK = 'typVazbyDokl.polozkaSkl');
        else
          raise exception 'dPolFakUpdateUcty(): Nelze zm�nit ��et u nav�zan� polo�ky.';
        end if;
      end if;
  end if;

  if (NEW.typPolozkyK in ('typPolozky.odpocetZdd','typPolozky.odpocetZalohy')) then
      -- nacteni typu pohybu
      select typPohybuK into typPohyb from dDoklFak where idDoklFak = NEW.idDoklFak;

      -- vydana faktura
      if (typPohyb = 'typPohybu.prijem') then
        -- meni se ucet zakladu
        if (compareId(NEW.idZklDalUcet, OLD.idZklDalUcet) = false) then
          -- polozka odpocitava zdd
          if (NEW.typPolozkyK = 'typPolozky.odpocetZdd') then
            for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                compareId((select idProtiUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idZklDalUcet) = false)
            loop
                execute dNastavUcetDokladu(0, idDokl, 1, NEW.idZklDalUcet);
            end loop;
          end if;
          -- polozka odpocitava zalohu
          if (NEW.typPolozkyK = 'typPolozky.odpocetZalohy') then
            for idDokl in (select idDoklInt from dDoklInt where compareId(idProtiUcet, NEW.idZklDalUcet) = false and idDoklInt in
                (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.uhradaZALOHY' and idTabA in
                  (select idTabB from dVazebTab where idTabA = NEW.idPolFak and typVazbyK = 'typVazbyDokl.odpocetZALOHY')))
            loop
                execute dNastavUcetDokladu(1, idDokl, 1, NEW.idZklDalUcet);
            end loop;
          end if;
        end if;    -- konec zmena uctu zakladu

        -- meni se ucet dane
        if (compareId(NEW.idDphDalUcet, OLD.idDphDalUcet) = false) then
          -- polozka odpocitava zdd
          if (NEW.typPolozkyK = 'typPolozky.odpocetZdd') then
            -- dph je zakladni
            if (NEW.typSzbDphK = 'typSzbDph.dphZakl') then
              for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                compareId((select idDphZaklUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idDphDalUcet) = false)
              loop
                  execute dNastavUcetDokladu(0, idDokl, 2, NEW.idDphDalUcet);
              end loop;
            end if;   -- konec dph zakladni

            -- dph je snizena
            if (NEW.typSzbDphK = 'typSzbDph.dphSniz') then
              for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                    compareId((select idDphSnizUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idDphDalUcet) = false)
              loop
                  execute dNastavUcetDokladu(0, idDokl, 3, NEW.idDphDalUcet);
              end loop;
            end if;-- konec dph snizena
          end if;  -- konec odpocet zdd
        end if;    -- konec zmena uctu dane

      end if;      -- konec jedna se o vydanou fakturu

      --------------------------------------------------------------------------
      -- prijata faktura
      if (typPohyb = 'typPohybu.vydej') then
        -- meni se ucet zakladu
        if (compareId(NEW.idZklMdUcet, OLD.idZklMdUcet) = false) then
          -- polozka odpocitava zdd
          if (NEW.typPolozkyK = 'typPolozky.odpocetZdd') then
            for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                compareId((select idProtiUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idZklMdUcet) = false)
            loop
                execute dNastavUcetDokladu(0, idDokl, 1, NEW.idZklMdUcet);
            end loop;
          end if;

          -- polozka odpocitava zalohu
          if (NEW.typPolozkyK = 'typPolozky.odpocetZalohy') then
            for idDokl in (select idDoklInt from dDoklInt where compareId(idProtiUcet, NEW.idZklMdUcet) = false and idDoklInt in
                (select idTabB from dVazebTab where typVazbyK = 'typVazbyDokl.uhradaZALOHY' and idTabA in
                  (select idTabB from dVazebTab where idTabA = NEW.idPolFak and typVazbyK = 'typVazbyDokl.odpocetZALOHY')))
            loop
                execute dNastavUcetDokladu(1, idDokl, 1, NEW.idZklMdUcet);
            end loop;
          end if;
        end if;    -- konec zmena uctu zakladu

        -- meni se ucet dane
        if (compareId(NEW.idDphMdUcet, OLD.idDphMdUcet) = false) then
          -- polozka odpocitava zdd
          if (NEW.typPolozkyK = 'typPolozky.odpocetZdd') then
            -- dph je zakladni
            if (NEW.typSzbDphK = 'typSzbDph.dphZakl') then
              for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                compareId((select idDphZaklUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idDphMdUcet) = false)
              loop
                  execute dNastavUcetDokladu(0, idDokl, 2, NEW.idDphMdUcet);
              end loop;

            end if;   -- konec dph zakladni

            -- dph je snizena
            if (NEW.typSzbDphK = 'typSzbDph.dphSniz') then
              for idDokl in (select idTabB from dVazebTab as V where typVazbyK = 'typVazbyDokl.odpocetZDD' and idTabA = NEW.idPolFak and
                    compareId((select idDphSnizUcet from dDoklFak where idDoklFak = V.idTabB), NEW.idDphMdUcet) = false)
              loop
                  execute dNastavUcetDokladu(0, idDokl, 3, NEW.idDphMdUcet);
              end loop;
            end if;-- konec dph snizena
          end if;  -- konec odpocet zdd
        end if;    -- konec zmena uctu dane

      end if;      -- konec jedna se o prijatou fakturu

  end if;       -- konec podminky na typ dokladu

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolfakupdateucty() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION dpolskladupdateucty() RETURNS trigger
    AS $$
declare nelzeZmena int;
begin
  if ((select typPohybuK from dDoklSklad where idDoklSklad = NEW.idDoklSklad) = 'typPohybu.prijem') then
    if (compareId(NEW.idZklDalUcet, OLD.idZklDalUcet) = false and NEW.kopZklDalUcet = false) then
      select count(*) into nelzeZmena from dPolFak where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and kopZklMdUcet = true and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
      if (nelzeZmena = 0) then
        update dPolFak set idZklMdUcet = NEW.idZklDalUcet where compareId(idZklMdUcet, NEW.idZklDalUcet) = false and
          idPolFak in (select idTabA from dVazebTab where idTabB = NEW.idPolSklad and typVazbyK = 'typVazbyDokl.polozkaSkl');
      else
        raise exception 'dPolSkladUpdateUcty(): Nelze zm�nit ��et u nav�zan� polo�ky.';
      end if;
    end if;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.dpolskladupdateucty() OWNER TO %firmaRole%;


-----------------------------------------------------------------------
-- Odstraneni starych zamku

ALTER TABLE ddoklfak DROP COLUMN zamek;
ALTER TABLE ddoklint DROP COLUMN zamek;
ALTER TABLE ddoklsklad DROP COLUMN zamek;
ALTER TABLE ddoklobch DROP COLUMN zamek;


-----------------------------------------------------------------------
-- Zamknutí období nastaví zámek do všech dokladů

CREATE OR REPLACE FUNCTION wzamkynastavzamky()
  RETURNS trigger AS $$
DECLARE
  _query character varying;
  _cond character varying;
  _condOldZamek character varying;
  _condPlatiOd character varying;
  _condPlatiDo character varying;
  _modulyLen integer;
  _modulyDoklFak character varying;
  _modulyDoklInt character varying;
  _modulyDoklObch character varying;
  _condModulyDoklFak character varying;
  _condModulyDoklInt character varying;
  _condModulyDoklSklad character varying;
  _condModulyDoklObch character varying;
BEGIN
  IF (
--      ((NEW.zamekk = 'zamek.zamceno') AND
--       (OLD.zamekk = 'zamek.otevreno' OR OLD.zamekk = 'zamek.prohlednuto' OR OLD.zamekk = 'zamek.polozamceno'))
--        OR
--      ((NEW.zamekk = 'zamek.polozamceno') AND
--       (OLD.zamekk = 'zamek.otevreno' OR OLD.zamekk = 'zamek.prohlednuto' OR OLD.zamekk = 'zamek.zamceno'))
      NEW.zamekk = 'zamek.zamceno' OR NEW.zamekk = 'zamek.polozamceno'
      ) THEN

    -- Omezeni na zaznamy u nichz lze zamek zprisnit
    IF (NEW.zamekk = 'zamek.polozamceno') THEN
      _condOldZamek = '(zamekk IS NULL OR (zamekk <> ''' || NEW.zamekk || ''' AND zamekk <> ''zamek.zamceno''))';
    ELSE
      _condOldZamek = '(zamekk IS NULL OR zamekk <> ''' || NEW.zamekk || ''')';
    END IF;

    -- Omezeni casoveho obdobi pro zamykani
    IF (NEW.platioddata IS NOT NULL) THEN
      _condPlatiOd = '(datvyst > ''' || NEW.platioddata || ''' OR datvyst = ''' || NEW.platioddata || ''')';
      _cond = _condPlatiOd  || ' AND ' || _condOldZamek;
    ELSE
      _condPlatiOd = NULL;
    END IF;

    IF (NEW.platidodata IS NOT NULL) THEN
      _condPlatiDo = '(datvyst < ''' || NEW.platidodata || ''' OR datvyst = ''' || NEW.platidodata || ''')';
      _cond = _condPlatiDo  || ' AND ' || _condOldZamek;
    ELSE
      _condPlatiDo = NULL;
    END IF;

    IF (_condPlatiOd IS NOT NULL AND _condPlatiDo IS NOT NULL) THEN
      _cond = _condPlatiOd || ' AND ' || _condPlatiDo || ' AND ' || _condOldZamek;
    END IF;

    -- Omezeni na moduly
    -- 1) dDoklFak
    _modulyDoklFak = '';
    IF (NEW.modulfav = TRUE) THEN
      _modulyDoklFak = _modulyDoklFak || ',''FAV''';
    END IF;
    IF (NEW.modulfap = TRUE) THEN
      _modulyDoklFak = _modulyDoklFak || ',''FAP''';
    END IF;
    IF (NEW.modulphl = TRUE) THEN
      _modulyDoklFak = _modulyDoklFak || ',''PHL''';
    END IF;
    IF (NEW.modulzav = TRUE) THEN
      _modulyDoklFak = _modulyDoklFak || ',''ZAV''';
    END IF;

    _modulyLen = char_length(_modulyDoklFak);
    IF (_modulyLen > 0) THEN
      _modulyLen = _modulyLen - 1;
      _modulyDoklFak = substring(_modulyDoklFak from 2 for _modulyLen);
      _condModulyDoklFak = 'modul IN (' || _modulyDoklFak || ')';
    ELSE
      _condModulyDoklFak = NULL;
    END IF;

    -- 2) dDoklInt
    _modulyDoklInt = '';
    IF (NEW.modulpok = TRUE) THEN
      _modulyDoklInt = _modulyDoklInt || ',''POK''';
    END IF;
    IF (NEW.modulban = TRUE) THEN
      _modulyDoklInt = _modulyDoklInt || ',''BAN''';
    END IF;
    IF (NEW.modulint = TRUE) THEN
      _modulyDoklInt = _modulyDoklInt || ',''INT''';
    END IF;

    _modulyLen = char_length(_modulyDoklInt);
    IF (_modulyLen > 0) THEN
      _modulyLen = _modulyLen - 1;
      _modulyDoklInt = substring(_modulyDoklInt from 2 for _modulyLen);
      _condModulyDoklInt = 'modul IN (' || _modulyDoklInt || ')';
    ELSE
      _condModulyDoklInt = NULL;
    END IF;

    -- 3) dDoklSklad
    IF (NEW.modulskl = TRUE) THEN
      _condModulyDoklSklad = 'modul = ''SKL''';
    ELSE
      _condModulyDoklSklad = NULL;
    END IF;

    -- 4) dDoklObch
    _modulyDoklObch = '';
    IF (NEW.modulppp = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''PPP''';
    END IF;
    IF (NEW.modulppv = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''PPV''';
    END IF;
    IF (NEW.modulnap = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''NAP''';
    END IF;
    IF (NEW.modulnav = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''NAV''';
    END IF;
    IF (NEW.modulobp = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''OBP''';
    END IF;
    IF (NEW.modulobv = TRUE) THEN
      _modulyDoklObch = _modulyDoklObch || ',''OBV''';
    END IF;

    _modulyLen = char_length(_modulyDoklObch);
    IF (_modulyLen > 0) THEN
      _modulyLen = _modulyLen - 1;
      _modulyDoklObch = substring(_modulyDoklObch from 2 for _modulyLen);
      _condModulyDoklObch = 'modul IN (' || _modulyDoklObch || ')';
    ELSE
      _condModulyDoklObch = NULL;
    END IF;

    -- Provedeni dotazu
    -- 1) dDoklFak
    IF (_condModulyDoklFak IS NOT NULL) THEN
      _query = 'UPDATE ddoklfak SET zamekk = ''' || NEW.zamekk || ''', lastupdate = now() ' ||
         'WHERE ' || _condModulyDoklFak || ' AND ' || _cond || ';';
      RAISE NOTICE 'Query is %', _query;
      EXECUTE _query;
    END IF;

    -- 2) dDoklInt
    IF (_condModulyDoklInt IS NOT NULL) THEN
      _query = 'UPDATE ddoklint SET zamekk = ''' || NEW.zamekk || ''', lastupdate = now() ' ||
         'WHERE ' || _condModulyDoklInt || ' AND ' || _cond || ';';
      RAISE NOTICE 'Query is %', _query;
      EXECUTE _query;
    END IF;

    -- 3) dDoklSklad
    IF (_condModulyDoklSklad IS NOT NULL) THEN
      _query = 'UPDATE ddoklsklad SET zamekk = ''' || NEW.zamekk || ''', lastupdate = now() ' ||
         'WHERE ' || _condModulyDoklSklad || ' AND ' || _cond || ';';
      RAISE NOTICE 'Query is %', _query;
      EXECUTE _query;
    END IF;

    -- 4) dDoklObch
    IF (_condModulyDoklObch IS NOT NULL) THEN
      _query = 'UPDATE ddoklobch SET zamekk = ''' || NEW.zamekk || ''', lastupdate = now() ' ||
         'WHERE ' || _condModulyDoklObch || ' AND ' || _cond || ';';
      RAISE NOTICE 'Query is %', _query;
      EXECUTE _query;
    END IF;

  END IF;

  RETURN NEW;
END;
$$ LANGUAGE plpgsql;

ALTER FUNCTION public.wzamkynastavzamky() OWNER TO %firmaRole%;

CREATE TRIGGER wzamkynastavzamky
  AFTER UPDATE ON wzamky
  FOR EACH ROW
  EXECUTE PROCEDURE wzamkynastavzamky();


-----------------------------------------------------------------------
-- Bug 16138 – [WinStrom] Implementace zámků pro modul majetek
-----------------------------------------------------------------------

ALTER TABLE mmajetek
  ADD COLUMN zamekk character varying(50) default 'zamek.otevreno';

UPDATE mmajetek SET zamekk = 'zamek.zamceno' WHERE zamek = true;


insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 163, 't');