--------------------------------------------------------------------------------
-- Bug 18198 – [WinStrom] Změnit typ u počtu balení z integer na numeric
--------------------------------------------------------------------------------

/* Enable/disable all the triggers in database */
CREATE OR REPLACE FUNCTION fn_triggerall(DoEnable boolean) RETURNS integer AS
$BODY$
DECLARE
mytables RECORD;
_trg_count integer;
_pg_version text;
BEGIN
  select count(*) into _trg_count from pg_attribute
  where attrelid = 'pg_class'::regclass
      and attnum > 0 and not attisdropped and attname = 'reltriggers';

  -- PostgreSQL 8.3
  IF _trg_count > 0 THEN
    FOR mytables IN SELECT relname FROM pg_class WHERE reltriggers > 0 AND NOT relname LIKE 'pg_%'
    LOOP
      IF DoEnable THEN
        EXECUTE 'ALTER TABLE ' || mytables.relname || ' ENABLE TRIGGER ALL';
      ELSE
        EXECUTE 'ALTER TABLE ' || mytables.relname || ' DISABLE TRIGGER ALL';
      END IF;
    END LOOP;
  ELSE
    select count(*) into _trg_count from pg_attribute
    where attrelid = 'pg_class'::regclass
       and attnum > 0 and not attisdropped and attname = 'relhastriggers';

    -- PostgreSQL 8.4
    IF _trg_count > 0 THEN
      FOR mytables IN SELECT relname FROM pg_class WHERE relhastriggers = true AND NOT relname LIKE 'pg_%'
      LOOP
        IF DoEnable THEN
          EXECUTE 'ALTER TABLE ' || mytables.relname || ' ENABLE TRIGGER ALL';
        ELSE
          EXECUTE 'ALTER TABLE ' || mytables.relname || ' DISABLE TRIGGER ALL';
        END IF;
      END LOOP;
    ELSE
      SELECT version() INTO _pg_version;
      raise exception 'fn_triggerall neni pro PostgreSQL % podporovan', _pg_version;
    END IF;
  END IF;

  RETURN 1;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION fn_triggerall(DoEnable boolean) OWNER TO %firmaRole%;
COMMENT ON FUNCTION fn_triggerall(DoEnable boolean) IS 'Enable/disable all the triggers in database';

--------------------------------------------------------------------------------

SELECT fn_triggerall(false);

--------------------------------------------------------------------------------

DROP VIEW dpolskladfBezVC;
DROP VIEW dpolskladf;
DROP VIEW dpolskladdoklad;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

ALTER TABLE ccenik ALTER COLUMN balenimj1 TYPE numeric(19,6);
ALTER TABLE ccenik ALTER COLUMN balenimj2 TYPE numeric(19,6);
ALTER TABLE ccenik ALTER COLUMN balenimj3 TYPE numeric(19,6);
ALTER TABLE ccenik ALTER COLUMN balenimj4 TYPE numeric(19,6);
ALTER TABLE ccenik ALTER COLUMN balenimj5 TYPE numeric(19,6);

ALTER TABLE ccenik ALTER COLUMN balenimj1 SET DEFAULT 0.0;
ALTER TABLE ccenik ALTER COLUMN balenimj2 SET DEFAULT 0.0;
ALTER TABLE ccenik ALTER COLUMN balenimj3 SET DEFAULT 0.0;
ALTER TABLE ccenik ALTER COLUMN balenimj4 SET DEFAULT 0.0;
ALTER TABLE ccenik ALTER COLUMN balenimj5 SET DEFAULT 0.0;


ALTER TABLE dpolfak ALTER COLUMN mnozbaleni TYPE numeric(19,6);
ALTER TABLE dpolint ALTER COLUMN mnozbaleni TYPE numeric(19,6);
ALTER TABLE dpolobch ALTER COLUMN mnozbaleni TYPE numeric(19,6);
ALTER TABLE dpolsklad ALTER COLUMN mnozbaleni TYPE numeric(19,6);

ALTER TABLE dpolfak ALTER COLUMN mnozbaleni SET DEFAULT 0.0;
ALTER TABLE dpolint ALTER COLUMN mnozbaleni SET DEFAULT 0.0;
ALTER TABLE dpolobch ALTER COLUMN mnozbaleni SET DEFAULT 0.0;
ALTER TABLE dpolsklad ALTER COLUMN mnozbaleni SET DEFAULT 0.0;


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

CREATE OR REPLACE VIEW dpolskladdoklad AS
 SELECT p.idpolsklad, p.lastupdate, p.autozaokr, p.autogen, p.baleniid, p.cenjednotka, p.cenamj, p.cenamjnakup, p.cenamjprodej, p.cisrad, p.datucto, p.datvyst, p.eankod, p.expirace, p.kod, p.kopclendph, p.kopdphdalucet, p.kopdphmducet, p.kopklice, p.kopstred, p.kopzakazku, p.kopzkldalucet, p.kopzklmducet, p.mnozbaleni, p.mnozmj, p.mnozmjplan, p.mnozmjreal, p.modul, p.nazev, p.objednatk, p.objem, p.poznam, p.proczakl, p.rezervovat, p.sarze, p.slevadokl, p.slevamnoz, p.slevapol, p.stavuzivk, p.storno, p.stornopol, p.sumdph, p.sumdphmen, p.sumzkl, p.sumzklmen, p.szbdph, p.typcenydphk, p.typpolozkyk, p.typszbdphk, p.typvypcenyk, p.ucetni, p.uplslevadokl, p.vyrcis, p.zaokrjakk, p.zaokrnak, p.idzkldalucet, p.iddphdalucet, p.idmjobjem, p.idmeny, p.idstred, p.idkarty, p.idcenik, p.idzklmducet, p.iddphmducet, p.idsazbydph, p.idclendph, p.idzakazky, p.idmj, p.iddoklsklad, p.idskladu, p.idfirmy, d.typpohybuk
   FROM dpolsklad p
   LEFT JOIN ddoklsklad d ON p.iddoklsklad = d.iddoklsklad;

ALTER TABLE dpolskladdoklad OWNER TO %firmaRole%;

---------------------------------------------------------------------------

CREATE OR REPLACE VIEW dpolskladf AS
 --J.Svik - pohled, který nabízí příjemky k vyskladnění
 SELECT p.idpolsklad, p.mnozmj - jevydano(p.idpolsklad) AS pocet, p.cenamj, p.datvyst, p.expirace, p.idzakazky, p.idskladu, p.idcenik, p.sarze, (-1) AS idvyrobnicislo, ( SELECT c.evidvyrcis
           FROM ccenik c
          WHERE c.idcenik = p.idcenik) AS evidvyrcis
   FROM dpolskladdoklad p
  WHERE p.typpohybuk::text = 'typPohybu.prijem'::text AND (p.mnozmj - jevydano(p.idpolsklad)) > 0::numeric AND (( SELECT c.evidvyrcis
           FROM ccenik c
          WHERE c.idcenik = p.idcenik)) = false AND p.storno <> true
UNION ALL
 SELECT p.idpolsklad, 1 AS pocet, p.cenamj, p.datvyst, p.expirace, p.idzakazky, p.idskladu, p.idcenik, p.sarze, v.idvyrobnicislo, ( SELECT c.evidvyrcis
           FROM ccenik c
          WHERE c.idcenik = p.idcenik) AS evidvyrcis
   FROM svyrobnicislo v
   LEFT JOIN dpolskladdoklad p ON p.idpolsklad = v.idpolskladprijem
  WHERE p.typpohybuk::text = 'typPohybu.prijem'::text AND (( SELECT c.evidvyrcis
      FROM ccenik c
     WHERE c.idcenik = p.idcenik)) = true AND NOT (v.idvyrobnicislo IN ( SELECT sfifovazby.idvyrobnicislo
      FROM sfifovazby
     WHERE sfifovazby.idpolskladprijem = p.idpolsklad)) AND p.storno <> true;
ALTER TABLE dpolskladf OWNER TO %firmaRole%;

---------------------------------------------------------------------------

-- View: dpolskladfbezVC
--DROP VIEW dpolskladfBezVC;
CREATE OR REPLACE VIEW dpolskladfBezVC AS
 SELECT p.idpolsklad, p.mnozmj - jevydano(p.idpolsklad) AS pocet, p.cenamj, p.datvyst, p.expirace, p.idzakazky, p.idskladu, p.idcenik, p.sarze, (-1) as idvyrobnicislo, false as evidVyrCis
   FROM dpolskladdoklad p
  WHERE p.typpohybuk::text = 'typPohybu.prijem'::text AND (p.mnozmj - jevydano(p.idpolsklad)) > 0::numeric AND p.storno <> true
/*UNION ALL
 SELECT p.idpolsklad, 1 AS pocet, p.cenamj, p.datvyst, p.expirace, p.idzakazky, p.idskladu, p.idcenik, p.sarze
   FROM dpolskladdoklad p
  WHERE p.typpohybuk::text = 'typPohybu.prijem'::text AND (( SELECT c.evidvyrcis
      FROM ccenik c
     WHERE c.idcenik = p.idcenik)) = true AND NOT (v.idvyrobnicislo IN ( SELECT sfifovazby.idvyrobnicislo
      FROM sfifovazby
     WHERE sfifovazby.idpolskladprijem = p.idpolsklad)) AND p.storno <> true*/;

ALTER TABLE dpolskladfBezVC OWNER TO %firmaRole%;

--------------------------------------------------------------------------------

SELECT fn_triggerall(true);


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 258, 't');