/* Drop constraint if exists */
CREATE OR REPLACE FUNCTION drop_constraint_if_exists(table_name character varying, constraint_name character varying)
  RETURNS void AS
$BODY$  
declare  
  _pocetConstraintu integer;
  _exception character varying;
begin
 select count(*) into _pocetConstraintu from pg_constraint where conname = constraint_name;
 if(_pocetConstraintu > 0) then
   execute 'Alter table '||table_name||' drop constraint '||constraint_name||';';
 end if;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION drop_constraint_if_exists(table_name character varying, constraint_name character varying) OWNER TO %firmaRole%;

/* Add constraint if not exists */
CREATE OR REPLACE FUNCTION add_constraint_if_not_exists(table_name character varying, constraint_name character varying, constraint_value character varying)
  RETURNS void AS
$BODY$  
declare  
  _pocetConstraintu integer;
begin
 select count(*) into _pocetConstraintu from pg_constraint where conname = constraint_name;
 if(_pocetConstraintu = 0) then
   execute 'Alter table '||table_name||' add constraint '||constraint_name||' '||constraint_value||';';
 end if;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION add_constraint_if_not_exists(table_name character varying, constraint_name character varying, constraint_value character varying) OWNER TO %firmaRole%;

/* Add column if not exists */
CREATE OR REPLACE FUNCTION add_column_if_not_exists(table_name character varying, column_name character varying, column_def character varying)
  RETURNS void AS
$BODY$  
declare  
  _pocetColumnu integer;
begin
 select count(*) into _pocetColumnu from pg_attribute as a where
  a.attname = column_name and 
  a.attrelid = (
        SELECT c.oid
        FROM pg_catalog.pg_class c
            LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
        WHERE c.relname ~ ('^('||table_name||')$')
            AND pg_catalog.pg_table_is_visible(c.oid)
    );
 if(_pocetColumnu = 0) then
   execute 'Alter table '||table_name||' add '||column_name||' '||column_def||';';
 end if;
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION add_column_if_not_exists(table_name character varying, column_name character varying, column_def character varying) OWNER TO %firmaRole%;


/* Pridani sloupce do typu dokladu */
select add_column_if_not_exists('dtypdokl', 'idtyporg', 'integer not null default 1');

update dtypdokl set idTypOrg = 1 where idTypOrg is null;

create index dTypDokl_TypOrg_index on dTypDokl using btree (IdTypOrg);

select add_constraint_if_not_exists('dtypdokl', 'dtypdokl_idtyporg_vazba', 'foreign key (idTypOrg) references utyporg(idTypOrg)');

/* Pridani sloupce do uucty */
select add_column_if_not_exists('uucty', 'idtyporg', 'integer not null default 1');

update uucty set idTypOrg = 1 where idTypOrg is null;

create index uucty_TypOrg_index on uucty using btree (IdTypOrg);

select add_constraint_if_not_exists('uucty', 'uucty_idtyporg_vazba', 'foreign key (idTypOrg) references utyporg(idTypOrg)');

/* Odstraneni unikatnosti kodu na ustducty */
select drop_constraint_if_exists('ustducty', 'ustducty_kod_key');
select add_constraint_if_not_exists('ustducty', 'ustducty_kod_typorg_key', 'UNIQUE (kod, idtyporg)');

/* Odstraneni unikatnosti kodu na typu dokladu */
select drop_constraint_if_exists('dtypdokl', 'dtypdokl_kod_key');
select add_constraint_if_not_exists('dtypdokl', 'dtypdokl_kod_modul_typorg_key', 'UNIQUE (kod, modul, idtyporg)');

/* Odstraneni unikatnosti kodu na uucty*/
select drop_constraint_if_exists('uucty', 'uucty_kod_key');
select add_constraint_if_not_exists('uucty', 'uucty_kod_typorg_key', 'UNIQUE (kod, idtyporg)');

insert into dbversion(iddbversion, dbversion, needreimport) values (nextval('seq_dbversion'), 26, 't');
