--------------------------------------------------------------------------------
-- Bug 19239 – [winstrom] 13. Pokročilá přístupová práva - možnost omezit/zakázat uživateli mazání/opravy dokladů.
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_wrole
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_wrole OWNER TO %firmaRole%;

create table wrole (
    idrole integer DEFAULT nextval('seq_wrole'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    standard boolean NOT NULL default false,
    "show" boolean DEFAULT true NOT NULL,
    pristupk character varying(50),
    uuid character varying(50),
    CONSTRAINT wrole_pkey PRIMARY KEY (idrole),
    CONSTRAINT wrole_kod_key UNIQUE (kod)
);
ALTER TABLE wrole OWNER TO %firmaRole%;

CREATE SEQUENCE seq_wpristprava
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_wpristprava OWNER TO %firmaRole%;

create table wpristprava (
    idpristprava integer DEFAULT nextval('seq_wpristprava'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    typeK character varying(50) NOT NULL,
    groupKey character varying(255) NOT NULL,
    uprcreate boolean NOT NULL default false,
    uprupdate boolean NOT NULL default false,
    uprdelete boolean NOT NULL default false,
    uprstorno boolean NOT NULL default false,
    idrole integer NOT NULL,
    CONSTRAINT wpristprava_pkey PRIMARY KEY (idpristprava),
    CONSTRAINT wpristprava_idrole_fk FOREIGN KEY (idrole)
        REFERENCES wrole(idrole)
);
ALTER TABLE wpristprava OWNER TO %firmaRole%;

CREATE INDEX wpristprava_idrole_index
  ON wpristprava
  USING btree
  (idrole);

create or replace function upgrade() returns void as
$BODY$
begin
    -- idstatuleg je konstantni po celou dobu trvani firmy
    if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'CZ_CZ')) then
        INSERT INTO wrole(idrole, lastupdate, kod, nazev, standard, pristupK, uuid)
        VALUES (nextval('seq_wrole'), now(), 'ADMIN', 'Administrátor', true, 'typPrav.admin', 'CZ_admin'),
            (nextval('seq_wrole'), now(), 'SUPERUZIVATEL', 'Super uživatel', true, 'typPrav.superUzivatel', 'CZ_superUzivatel'),
            (nextval('seq_wrole'), now(), 'MZDOVYUCETNI', 'Mzdový účetní', true, 'typPrav.mzdovyUcetni', 'CZ_mzdovyUcetni'),
            (nextval('seq_wrole'), now(), 'UCETNI', 'Účetní', true, 'typPrav.ucetni', 'CZ_ucetni'),
            (nextval('seq_wrole'), now(), 'OBCHODNIK', 'Obchodník', true, 'typPrav.obchodnik', 'CZ_obchodnik'),
            (nextval('seq_wrole'), now(), 'SKLADNIK', 'Skladník', true, 'typPrav.skladnik', 'CZ_skladnik'),
            (nextval('seq_wrole'), now(), 'SKLADNIKSPOKLADNOU', 'Skladník s pokladnou', true, 'typPrav.skladnikSPokladnou', 'CZ_skladnikSPokladnou'),
            (nextval('seq_wrole'), now(), 'UZIVATEL', 'Standardní uživatel', true, 'typPrav.uzivatel', 'CZ_uzivatel'),
            (nextval('seq_wrole'), now(), 'JENCIST', 'Jen číst', true, 'typPrav.jenCist', 'CZ_jenCist'),
            (nextval('seq_wrole'), now(), 'ZABLOKOVAN', 'Neaktivní účet', true, 'typPrav.zablokovan', 'CZ_zablokovan');
    else
        INSERT INTO wrole(idrole, lastupdate, kod, nazev, standard, pristupK, uuid)
        VALUES (nextval('seq_wrole'), now(), 'ADMIN', 'Administrátor', true, 'typPrav.admin', 'SK_admin'),
            (nextval('seq_wrole'), now(), 'SUPERUZIVATEL', 'Super užívateľ', true, 'typPrav.superUzivatel', 'SK_superUzivatel'),
            (nextval('seq_wrole'), now(), 'MZDOVYUCTOVNIK', 'Mzdový účtovník', true, 'typPrav.mzdovyUcetni', 'SK_mzdovyUctovnik'),
            (nextval('seq_wrole'), now(), 'UCTOVNIK', 'Účtovník', true, 'typPrav.ucetni', 'SK_uctovnik'),
            (nextval('seq_wrole'), now(), 'OBCHODNIK', 'Obchodník', true, 'typPrav.obchodnik', 'SK_obchodnik'),
            (nextval('seq_wrole'), now(), 'SKLADNIK', 'Skladník', true, 'typPrav.skladnik', 'SK_skladnik'),
            (nextval('seq_wrole'), now(), 'SKLADNIKSPOKLADNOU', 'Skladník s pokladňou', true, 'typPrav.skladnikSPokladnou', 'SK_skladnikSPokladnou'),
            (nextval('seq_wrole'), now(), 'UZIVATEL', 'Štandardný užívateľ', true, 'typPrav.uzivatel', 'SK_uzivatel'),
            (nextval('seq_wrole'), now(), 'LENCITAT', 'Len čítať', true, 'typPrav.jenCist', 'SK_lenCitat'),
            (nextval('seq_wrole'), now(), 'ZABLOKOVAN', 'Neaktívny účet', true, 'typPrav.zablokovan', 'SK_zablokovan');
    end if;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION upgrade() OWNER TO %firmaRole%;

select upgrade();
drop function upgrade();

alter table wuzivatel add column idrole integer;

alter table wuzivatel add constraint wuzivatel_idrole_fk foreign key (idrole)
    references wrole(idrole);

create index wUzivatel_idrole_index
    ON wUzivatel
    USING btree (idrole);

create or replace function upgrade() returns void as
$BODY$
declare
    _uzivatel record;
    _idrole integer;
begin
    for _uzivatel in select * from wuzivatel loop
        -- pro uzivatele nalezneme prislusnou systemovou roli
        select idrole into _idrole from wrole r where r.pristupk = _uzivatel.pristupk;
        update wuzivatel set idrole = _idrole where iduzivatel = _uzivatel.iduzivatel;
    end loop;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION upgrade() OWNER TO %firmaRole%;

select upgrade();
drop function upgrade();

alter table wuzivatel drop column pristupk;
alter table wuzivatel alter column idrole set not null;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 311, 't');
