--------------------------------------------------------------------------------
-- Bug 20323 – [WinStrom] "Předrezervace"
--------------------------------------------------------------------------------

-- uprava tabulky rezervaci
ALTER TABLE srezervace ALTER COLUMN datumod TYPE timestamp without time zone;
ALTER TABLE srezervace ADD COLUMN skladmj numeric(19,6) NOT NULL DEFAULT 0.0;

update srezervace set skladmj = mnozstvi; -- TODO je tohle to co chceme?


-- uprave funkce zjistujici stav na sklade
CREATE OR REPLACE FUNCTION dgetstavnasklade(i_skladkartaid integer, i_datum date, i_polskladid integer, i_firmaid integer, i_sarze character varying, i_expirace date)
  RETURNS numeric AS $$
declare
  _sklad integer;
  _cenik integer;
  _stavMj numeric(19,6);
  _firmaRezerMj numeric(19,6);
  _celkemRezerMj numeric(19,6);
begin
  SELECT dgetstavnasklade(i_skladkartaid, i_datum, i_polskladid, i_sarze, i_expirace) INTO _stavMj;
 
  -- vymazeme jiz neplatne rezervace
  delete from srezervace where datumdo is not null and datumdo < date(now());
  -- nacteme si info ze skladove karty
  SELECT idCenik, idTypDokl INTO _cenik, _sklad FROM sKarty WHERE idKarty = i_skladkartaid;
  -- zjistime kolik je celkem rezervovano
  select coalesce(sum(skladmj), 0) into _celkemRezerMj from srezervace where idcenik = _cenik and idsklad = _sklad;
  
  IF (i_firmaid > 0) THEN
    -- zjistime kolik ma firma rezervovano
    select coalesce(sum(skladmj), 0) into _firmaRezerMj from srezervace where idfirmy = i_firmaid and idcenik = _cenik and idsklad = _sklad;
  ELSE
    _firmaRezerMj := 0;
  END IF;
  
  -- spocteme kolik mu muzeme dat
  _stavMj := minValue(_stavMj, maxValue(_stavMj - _celkemRezerMj, 0) + _firmaRezerMj);
  
  RETURN _stavMj;
end
$$ LANGUAGE plpgsql;
ALTER FUNCTION dgetstavnasklade(integer, date, integer, integer, character varying, date) OWNER TO %firmaRole%;


-- uprava odrezervovavajici funkce pri vydeji
CREATE OR REPLACE FUNCTION odrezervuj(_idFirmy integer, _idCenik integer, _idSkladu integer, vyskladni numeric(19,6))
  RETURNS void AS $$
declare
  _idRezer integer;
  _mnozstvi numeric(19,6);
  _skladMj numeric(19,6);
  _vyskl numeric(19,6);
begin
  _vyskl = vyskladni;
  
  perform putglotxvariable('rezerDisableTrigger', '1');
  for _idRezer, _mnozstvi, _skladMj in select idRezervace, mnozstvi, skladmj from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu order by idrezervace loop
    if _vyskl < _skladMj or _skladMj <> _mnozstvi then
      update sRezervace set mnozstvi = (mnozstvi-_vyskl), skladmj = (skladmj-_vyskl) where idRezervace = _idRezer;
      
      perform removeglotxvariable('rezerDisableTrigger');
      return;
    else
      delete from sRezervace where idRezervace = _idRezer;
      
      _vyskl = _vyskl - _mnozstvi;
    end if;
  end loop;
  
  if (_vyskl > 0) then        -- nemelo by nastat, ale pokud zbyde, odrezervujeme i to co nebylo rezervoveny primo na sklade
    for _idRezer, _mnozstvi in select idRezervace, mnozstvi from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu order by idrezervace loop
      if _vyskl < _mnozstvi then
        update sRezervace set mnozstvi = (mnozstvi-_vyskl) where idRezervace = _idRezer;
      
        perform removeglotxvariable('rezerDisableTrigger');
        return;
      else
        delete from sRezervace where idRezervace = _idRezer;
      
        _vyskl = _vyskl - _mnozstvi;
      end if;
    end loop;
  end if;
  
  perform removeglotxvariable('rezerDisableTrigger');
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.odrezervuj(integer, integer, integer, numeric) OWNER TO %firmaRole%;


-- uprava triggeru na skladove polozce
CREATE OR REPLACE FUNCTION dpolskladupdaterezer()
  RETURNS trigger AS $$
declare
  zmena numeric(19,6);
  _idFirmy integer;
  _typPohybuK varchar(50);
begin
  zmena = 0;
  if (tg_op = 'UPDATE') then
    if (OLD.mnozMj = NEW.mnozMj and compareid(OLD.idCenik, NEW.idCenik) and compareid(OLD.idSkladu, NEW.idSkladu)) then
      return NEW;         -- pokud nedoslo ke zmene ktera se nas netyka, ukoncime aby jsme nezpomalovali sklad
    end if;
  
    zmena = OLD.mnozMj;
  end if;
  
  zmena = NEW.mnozMj - zmena;
  select idfirmy, typPohybuK into _idFirmy, _typPohybuK from ddoklsklad where idDoklSklad = NEW.idDoklSklad;

  if (zmena > 0 and _typPohybuK = 'typPohybu.vydej') then
    perform odrezervuj(_idFirmy, NEW.idCenik, NEW.idSkladu, zmena);
  end if;
  if (zmena <> 0 and _typPohybuK = 'typPohybu.prijem') then
    perform srezervujzprijemky(NEW.idCenik, NEW.idSkladu, zmena);
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolskladupdaterezer() OWNER TO %firmaRole%;


-- funkce rezervujici z prijemky
CREATE OR REPLACE FUNCTION srezervujzprijemky(_idCenik integer, _idSkladu integer, prijato numeric(19,6))
  RETURNS void AS $$
declare
  _zbyva numeric(19,6);
  _uber numeric(19,6);
  _rezer record;
  _idUcetObd integer;
  _sklad numeric(19,6);
begin
  _zbyva = prijato;
  if (prijato > 0) then
    for _rezer in select * from sRezervace where idCenik = _idCenik and idSklad = _idSkladu order by datumod ASC loop
      if (_rezer.mnozstvi > _rezer.skladmj) then
        _uber = minvalue(_zbyva, _rezer.mnozstvi - _rezer.skladmj);
        _zbyva = _zbyva - _uber;
        
        update sRezervace set skladmj = skladmj + _uber where idrezervace = _rezer.idrezervace;
        
        if (_zbyva = 0) then
          return;
        end if;
      end if;
    end loop;
  else
    select idUcetObdobi into _idUcetObd from nUcetObdobi where platioddata <= date(now()) and platidodata >= date(now()); 
    select stavmj into _sklad from skarty where iducetobdobi = _idUcetObd and idtypdokl = _idSkladu and idcenik = _idCenik;
    select sum(skladmj) into _zbyva from srezervace where idCenik = _idCenik and idSklad = _idSkladu;
    
    if (_zbyva > _sklad) then
      _zbyva = _zbyva - _sklad;
      
      for _rezer in select * from sRezervace where idCenik = _idCenik and idSklad = _idSkladu order by datumod DESC loop
        if (_rezer.skladmj > 0) then
          _uber = minvalue(_zbyva, _rezer.skladmj);
          _zbyva = _zbyva - _uber;
          
          update sRezervace set skladmj = skladmj - _uber where idrezervace = _rezer.idrezervace;
          
          if (_zbyva = 0) then
            return;
          end if;
        end if;
      end loop;
    end if;
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.srezervujzprijemky(integer, integer, numeric) OWNER TO %firmaRole%;


-- trigger resici smazani skladove polozky
CREATE OR REPLACE FUNCTION dpolskladdeleterezer()
  RETURNS trigger AS $$
begin
  if (OLD.mnozMj > 0) then
    perform srezervujzprijemky(OLD.idCenik, OLD.idSkladu, -OLD.mnozMj);
  end if;

  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolskladdeleterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek skladovych dokladů
CREATE TRIGGER dpolskladdeleterezer AFTER DELETE ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpolskladdeleterezer();


-- trigger pri update rezervace
CREATE OR REPLACE FUNCTION srezervaceupdaterezer()
  RETURNS trigger AS $$
declare
  zmena numeric(19,6);
  puvodni numeric(19,6);
  volnosklad numeric(19,6);
begin
  if (getglotxvariable('rezerDisableTrigger') is not null) then
    return NEW;
  end if;

  zmena = NEW.mnozstvi;
  puvodni = 0;
  if (tg_op = 'UPDATE') then
    if (NEW.idsklad = OLD.idsklad and NEW.idcenik = OLD.idcenik and NEW.mnozstvi = OLD.mnozstvi and NEW.datumod = OLD.datumod) then
      return NEW;
    end if;
  
    if (NEW.idsklad <> OLD.idsklad or NEW.idcenik <> OLD.idcenik) then          -- pokud menime zbozi na rezervaci
      if (OLD.skladmj > 0) then                                                 -- a nejake bylo na sklade
        perform srezervujzprijemky(OLD.idcenik, OLD.idsklad, OLD.skladmj);     -- muzeme ho zarezervovat nekomu jinemu
      end if;
    else
      puvodni = OLD.skladmj;
      zmena = NEW.mnozstvi - OLD.skladmj;
      if (NEW.mnozstvi > OLD.mnozstvi) then             -- mnozstvi se zvetsilo
        if (OLD.skladmj < OLD.mnozstvi) then            -- pokud na sklade bylo min nez zarezervovano, tak stejne nic na sklade nenajdem
          zmena = 0;
        end if;
      end if;
    end if;
  end if;
  
  if (zmena > 0) then
    select dgetstavnasklade(NEW.idcenik, NEW.idsklad, date(now()), -1, -1, null, null) into volnosklad;
    
    update srezervace set skladmj = puvodni + minvalue(mnozstvi-puvodni, volnosklad) where idrezervace = NEW.idrezervace;
  else
    if (zmena < 0) then
      update srezervace set skladmj = puvodni + zmena where idrezervace = NEW.idrezervace;
    
      perform srezervujzprijemky(NEW.idcenik, NEW.idsklad, -zmena);
    end if;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.srezervaceupdaterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku
CREATE TRIGGER srezervaceupdaterezer AFTER INSERT OR UPDATE ON srezervace
  FOR EACH ROW EXECUTE PROCEDURE srezervaceupdaterezer();


-- trigger pri smazani rezervace
CREATE OR REPLACE FUNCTION srezervacedeleterezer()
  RETURNS trigger AS $$
begin
  if (getglotxvariable('rezerDisableTrigger') is not null) then
    return OLD;
  end if;
  
  perform srezervujzprijemky(OLD.idcenik, OLD.idsklad, OLD.skladmj);     -- muzeme ho zarezervovat nekomu jinemu
  
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.srezervacedeleterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku
CREATE TRIGGER srezervacedeleterezer AFTER DELETE ON srezervace
  FOR EACH ROW EXECUTE PROCEDURE srezervacedeleterezer();


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 376, 't');
