------------------------------------------------------------------------------------------------------------------
-- Bug 21102 – [WinStrom] V převodce - výdejce - mohu dodatečně změnit v.č., ale do hotové příjemky se nepromítne
------------------------------------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION svyrobnicisloupdateprevodka()
  RETURNS trigger AS
$BODY$ 
declare
  _vazbaPrevod record;
  _polSkladPrijemPrevod record;
begin
  /* Vytvoreni novych vyrobnich cisel pro vyrobni cisla, ktera byla privazana */
  if(NEW.idPolSkladVydej is not null and OLD.idPolSkladVydej is null) then
    select * into _vazbaPrevod from dvazebtab where idtabA = NEW.idPolSkladVydej and modulA = 'SKL' and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod';
    if(_vazbaPrevod is not null) then
      select * into _polSkladPrijemPrevod from dpolsklad where idPolSklad = _vazbaPrevod.idTabB;
      insert into svyrobnicislo(kod, idcenik, idtypdokl, 
          idpolskladprijem, idpolskladvydej, idpolfakprijem, idpolfakvydej, idpolintprijem, idpolintvydej, idvyrobnicislohlav)
        values(NEW.kod, NEW.idcenik, _polSkladPrijemPrevod.idSkladu,
          _polSkladPrijemPrevod.idPolSklad, null, null, null, null, null, NEW.idvyrobnicislohlav);
    end if;
  end if;

  /* Smazani vyrobnich cisel, ktere byli vygenerovany pro odvazana vyrobni cisla */
  if(NEW.idPolSkladVydej is null and OLD.idPolSkladVydej is not null) then
    select * into _vazbaPrevod from dvazebtab where idtabA = OLD.idPolSkladVydej and modulA = 'SKL' and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod';
    if(_vazbaPrevod is not null) then
      delete from svyrobnicislo where idvyrobnicislohlav = NEW.idVyrobniCisloHlav and idpolskladprijem = _vazbaPrevod.idTabB 
        and idPolSkladVydej is null and idPolFakVydej is null and idPolIntVydej is null;
    end if;
  end if;

  return new;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicisloupdateprevodka() OWNER TO %firmaRole%;

CREATE TRIGGER svyrobnicisloupdateprevodka
  BEFORE UPDATE
  ON svyrobnicislo
  FOR EACH ROW
  EXECUTE PROCEDURE svyrobnicisloupdateprevodka();


-------------------------------------------------------------------------
-- Bug 21144 – [WinStrom] Položky skladových dokladů - úprava kvůli v.č.
-------------------------------------------------------------------------
-- Function: svyrobnicisloinsertupdatezavyrobneno()
-- DROP FUNCTION svyrobnicisloinsertupdatezavyrobneno();
CREATE OR REPLACE FUNCTION svyrobnicisloinsertupdatezavyrobneno()
  RETURNS trigger AS
$BODY$
declare
  _pocetVyrCisel integer;
  _mnozMj numeric(19,6);
  _vyrobniCislaOK boolean;
  _oldId integer;
  _newId integer;
begin
  -- prijemka
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolSkladPrijem;
  else
    _oldId = OLD.idPolSkladPrijem;
    _newId = NEW.idPolSkladPrijem;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolSkladPrijem = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolSklad where idPolSklad = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolSklad set VyrobniCislaOk = false where idPolSklad = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolSklad set VyrobniCIslaOK = true where idPolSklad = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;
  -- vydejka
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolSkladVydej;
  else
    _oldId = OLD.idPolSkladVydej;
    _newId = NEW.idPolSkladVydej;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolSkladVydej = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolSklad where idPolSklad = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolSklad set VyrobniCislaOk = false where idPolSklad = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolSklad set VyrobniCIslaOK = true where idPolSklad = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;

  -- prijata faktura
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolFakPrijem;
  else
    _oldId = OLD.idPolFakPrijem;
    _newId = NEW.idPolFakPrijem;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolFakPrijem = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolFak where idPolFak = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolFak set VyrobniCislaOk = false where idPolFak = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolFak set VyrobniCislaOK = true where idPolFak = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;
  -- vydana faktura
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolFakVydej;
  else
    _oldId = OLD.idPolFakVydej;
    _newId = NEW.idPolFakVydej;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolFakVydej = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolFak where idPolFak = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolFak set VyrobniCislaOk = false where idPolFak = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolFak set VyrobniCislaOK = true where idPolFak = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;

  -- vydajovy pokladni doklad
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolIntPrijem;
  else
    _oldId = OLD.idPolIntPrijem;
    _newId = NEW.idPolIntPrijem;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolIntPrijem = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolInt where idPolInt = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolInt set VyrobniCislaOk = false where idPolInt = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolInt set VyrobniCislaOK = true where idPolInt = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;
  -- prijmovy pokladni doklad
  if(tg_op = 'INSERT') then
    _oldId = null;
    _newId = NEW.idPolIntVydej;
  else
    _oldId = OLD.idPolIntVydej;
    _newId = NEW.idPolIntVydej;
  end if;
  if(_newId is not null and _oldId is null) or (_newId <> _oldId) then
    select count(*) into _pocetVyrCisel from svyrobniCislo where idPolIntVydej = _newId;
    select vyrobniCislaOK, mnozMj into _vyrobniCislaOK, _mnozMj from dPolInt where idPolInt = _newId;
    if(_mnozMj < 0.0) then
      _mnozMj = 0-_mnozMj;
    end if;
    if(_pocetVyrCisel <> _mnozMj and _vyrobniCislaOK = true) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolInt set VyrobniCislaOk = false where idPolInt = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
    if(_pocetVyrCisel = _mnozMj and _vyrobniCislaOK = false) then
      execute putglotxvariable('vypniSklad', 'true');
      update dPolInt set VyrobniCislaOK = true where idPolInt = _newId;
      execute removeglotxvariable('vypniSklad');
    end if;
  end if;
  RETURN NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicisloinsertupdatezavyrobneno() OWNER TO %firmaRole%;

insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 397, 't');