--------------------------------------------------------------------------------
-- Bug 23901 – [winstrom10] přiznání DPH par.92a
--------------------------------------------------------------------------------

-- zalozeni tabulky pro preneseni danove povinnosti
CREATE SEQUENCE seq_udphpren
  START WITH 1 
  INCREMENT BY 1
  NO MAXVALUE
  NO MINVALUE
  CACHE 1;
ALTER TABLE public.seq_udphpren OWNER TO %firmaRole%;

CREATE TABLE udphpren
(
  iddphpren integer NOT NULL DEFAULT nextval('seq_udphpren'::regclass),
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  kod character varying(20) NOT NULL,
  nazev character varying(255) NOT NULL,
  nazeva character varying(255),
  nazevb character varying(255),
  nazevc character varying(255),
  popis text,
  poznam text,
  "show" boolean NOT NULL DEFAULT true,
  platido integer NOT NULL DEFAULT 9999,
  platiod integer NOT NULL DEFAULT 0,
  uuid character varying(50),
  CONSTRAINT udphpren_pkey PRIMARY KEY (iddphpren),
  CONSTRAINT udphpren_kod_key UNIQUE (kod)
);
ALTER TABLE public.udphpren OWNER TO %firmaRole%;

-- data pro preneseni danove povinnosti jenom do ceske legislativy
create or replace function runupgrade()
    returns void as $$
declare
    stat character varying;
begin
    select a.uuid into stat from nnastav n  left join utyporg u on u.idtyporg = n.idtyporg left join astaty a on a.idstatu = u.idstatu
        where ((select max(platioddata) from nnastav) is null or (select max(platioddata) from nnastav) = platioddata);
    
    if (stat = 'CZ_CZ') then
        insert into udphpren (kod, nazev, uuid)
        values
        ('2618 00 00','Granulovaná struska (struskový písek) z výroby železa nebo oceli','CZ_2618_00_00'),
        ('2619 00','Struska, zpěněná struska (jiná než granulovaná struska), okuje a jiné odpady z výroby železa nebo oceli','CZ_2619_00'),
        ('2620','Struska, popel a zbytky (jiné než z výroby železa nebo oceli), obsahující kovy, arsen, nebo jejich sloučeniny','CZ_2620'),
        ('3915','Odpady, úlomky a odřezky z plastů ','CZ_3915'),
        ('4004 00 00','Odpady, úlomky a odřezky z kaučuku (jiné než z tvrdého kaučuku) a prášky a zrna z nich','CZ_4004_00_00'),
        ('4707','Sběrový papír, kartón nebo lepenka (odpad a výmět)','CZ_4707'),
        ('6310',' Použité nebo nové hadry, motouzy, šňůry, provazy a lana z textilních materiálů ve formě zbytků nebo výrobků vyřazených z používání','CZ_6310'),
        ('7001 00 10','Skleněné střepy a jiné skleněné odpady','CZ_7001_00_10'),
        ('7112','Odpad a šrot z drahých kovů nebo z kovů plátovaných drahými kovy, ostatní odpad a šrot obsahující drahé kovy nebo sloučeniny drahých kovů, používaný hlavně pro rekuperaci drahých kovů','CZ_7112'),
        ('7204','Odpad a šrot ze železa nebo oceli,  přetavený odpad ze železa nebo oceli v ingotech','CZ_7204'),
        ('7404 00','Měděný odpad a šrot','CZ_7404_00'),
        ('7503 00','Niklový odpad a šrot','CZ_7503_00'),
        ('7602 00','Hliníkový odpad a šrot','CZ_7602_00'),
        ('7802 00 00','Olověný odpad a šrot','CZ_7802_00_00'),
        ('7902 00 00','Zinkový odpad a šrot','CZ_7902_00_00'),
        ('8002 00 00','Cínový odpad a šrot','CZ_8002_00_00'),
        ('8101 97 00','Odpad a šrot z wolframu','CZ_8101_97_00'),
        ('8102 97 00','Odpad a šrot z molybdenu','CZ_8102_97_00'),
        ('8103 30 00','Odpad a šrot z tantalu','CZ_8103_30_00'),
        ('8104 20 00','Odpad a šrot z hořčíku','CZ_8104_20_00'),
        ('8105 30 00','Odpad a šrot z kobaltu ex ','CZ_8105_30_00'),
        ('8106 00 10','Odpad a šrot z bismutu','CZ_8106_00_10'),
        ('8107 30 00','Odpad a šrot z kadmia','CZ_8107_30_00'),
        ('8108 30 00','Odpad a šrot z titanu','CZ_8108_30_00'),
        ('8109 30 00','Odpad a šrot ze zirkonia','CZ_8109_30_00'),
        ('8110 20 00','Odpad a šrot z antimonu','CZ_8110_20_00'),
        ('8111 00 19','Odpad a šrot z manganu','CZ_8111_00_19'),
        ('8112 13 00','Odpad a šrot z berylia','CZ_8112_13_00'),
        ('8112 22 00','Odpad a šrot z chrómu','CZ_8112_22_00'),
        ('8112 52 00','Odpad a šrot z thallia','CZ_8112_52_00'),
        ('8112 92 21','Odpad a šrot z niobu (columbia),  gallia,  rhenia,  india,  vanadu a germania','CZ_8112_92_21'),
        ('8113 00 40','Odpad a šrot z cermetů','CZ_8113_00_40'),
        ('8548 10','Odpad a zbytky galvanických článků, baterií a elektrických akumulátorů,  nepoužitelné galvanické články, baterie a elektrické akumulátory.','CZ_8548_10');
    end if;
 end;
 $$ LANGUAGE plpgsql;
 ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();

-- pridani odkazu z ceniku na preneseni danove povinnosti
alter table ccenik 
    add column iddphpren integer,
    add constraint ccenik_iddphpren_fk foreign key (iddphpren)
        references udphpren(iddphpren) match simple
        on update no action on delete no action;

-- pridani odkazu z polozek faktur na preneseni danove povinnosti
alter table dpolfak
    add column iddphpren integer,
    add constraint dpolfak_iddphpren_fk foreign key (iddphpren)
        references udphpren(iddphpren) match simple
        on update no action on delete no action;

alter table dpolint
    add column iddphpren integer,
    add constraint dpolint_iddphpren_fk foreign key (iddphpren)
        references udphpren(iddphpren) match simple
        on update no action on delete no action;

alter table dpolobch
    add column iddphpren integer,
    add constraint dpolobch_iddphpren_fk foreign key (iddphpren)
        references udphpren(iddphpren) match simple
        on update no action on delete no action;

alter table dpolsklad
    add column iddphpren integer,
    add constraint dpolsklad_iddphpren_fk foreign key (iddphpren)
        references udphpren(iddphpren) match simple
        on update no action on delete no action;

-- pridani indexu
CREATE INDEX cCenik_IdDphPren_index ON cCenik USING btree (IdDphPren);
CREATE INDEX dPolFak_IdDphPren_index ON dPolFak USING btree (IdDphPren);
CREATE INDEX dPolInt_IdDphPren_index ON dPolInt USING btree (IdDphPren);
CREATE INDEX dPolObch_IdDphPren_index ON dPolObch USING btree (IdDphPren);
CREATE INDEX dPolSklad_IdDphPren_index ON dPolSklad USING btree (IdDphPren);

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 490, 't', %appVersion%, clock_timestamp());
