--------------------------------------------------------------------------------
-- Bug 23686 – [winstrom] - Cenotvorba - Analýza
-- Bug 23860 – [winstrom] - Ceník - Vizualizace ceníků dle odběratelů nebo skupin
--------------------------------------------------------------------------------

-- pridani chybejicich sloupcu
ALTER TABLE ccenhlad ADD COLUMN vsechnyfirmy boolean NOT NULL DEFAULT false;

ALTER TABLE codberatele ADD COLUMN idskupcen integer;

ALTER TABLE codberatele ADD CONSTRAINT codberatele_idskupcen FOREIGN KEY (idskupcen)
  REFERENCES cskupcen (idskupcen) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX cOdberatele_IdSkupCen_index ON cOdberatele USING btree (IdSkupCen);

-- vytvoreni funkci pro vypocet ceny
-- obecne zaokrouhlovani pouzivane v aplikaci
CREATE OR REPLACE FUNCTION zaokrouhli(vstup numeric, zaokrJakK varchar, zaokrNaK varchar)
  RETURNS numeric AS $$
declare
  nasob numeric;
  vysledek numeric;
begin
  if (zaokrNaK is null or zaokrNaK = 'zaokrNa.zadne') then
    return vstup;
  end if;
  
  nasob = (CASE zaokrNaK
    WHEN 'zaokrNa.tisiciny' THEN 1000
    WHEN 'zaokrNa.setiny'   THEN 100
    WHEN 'zaokrNa.desetiny' THEN 10
    WHEN 'zaokrNa.desitky'  THEN 0.1
    WHEN 'zaokrNa.stovky'   THEN 0.01
    WHEN 'zaokrNa.tisice'   THEN 0.001
    ELSE 1
  END);
  vysledek = vstup * nasob;
  
  vysledek = (CASE zaokrJakK
    WHEN 'zaokrJak.nahoru' THEN ceil(vysledek)
    WHEN 'zaokrJak.dolu'   THEN floor(vysledek)
    ELSE round(vysledek)
  END);
  
  return vysledek / nasob;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.zaokrouhli(numeric, varchar, varchar) OWNER TO %firmaRole%;

-- vypocet jedne konretni ceny podle zadanych parametru
CREATE OR REPLACE FUNCTION calccena(vychozi numeric, typVypCenyK varchar, procento numeric)
  RETURNS numeric AS $$
declare
  proc numeric;
  vysledek numeric;
begin
  proc = procento / 100;
  
  vysledek = (CASE typVypCenyK
    WHEN 'typVypCeny.marze' THEN vychozi * (1.0 + proc)
    WHEN 'typVypCeny.rabat' THEN vychozi / (1.0 - proc)
    WHEN 'typVypCeny.sleva' THEN vychozi * (1.0 - proc)
    ELSE null
  END);
  
  if vysledek is null then
    raise exception 'Neznamy zpusob vypoctu ceny "%"', typVypCenyK;
  end if;
  
  -- TODO mozna zaokrouhlit jako v aplikaci
  
  return vysledek;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.calccena(numeric, varchar, numeric) OWNER TO %firmaRole%;

-- vypocet cen pro cenik/cenovou hladinu/firmu
CREATE OR REPLACE FUNCTION calcceny(_idcenik integer, _idcenhlad integer, _idfirma integer, _idskupcen integer)
  RETURNS numeric[] AS $$
declare
  cenik record;
  cenhlad record;
  odberatelcena numeric;
begin
  -- otestujeme zda nemame cenu pro konretniho odberatele
  odberatelcena = null;
  if _idfirma is not null then      -- zkusime najit jestli pro firmu neexistuje prima odberatelska cena
    select prodejcena into odberatelcena from codberatele where idcenik = _idcenik and idfirmy = _idfirma limit 1;
  end if;
  if _idskupcen is not null and odberatelcena is null then  -- zkusime najit jestli pro skupinu cen neexistuje prima odberatelska cena, pouze pokud uz nebyla nalezena pro firmu
    select prodejcena into odberatelcena from codberatele where idcenik = _idcenik and idskupcen = _idskupcen limit 1;
  end if;
  
  select * into cenik from ccenik where idcenik = _idcenik;

  if (_idcenhlad is null) then
    return calcceny(cenik.nakupCena, cenik.cenazakl,
    array[cenik.cena2, cenik.cena3, cenik.cena4, cenik.cena5], array[cenik.limmnoz2, cenik.limmnoz3, cenik.limmnoz4, cenik.limmnoz5],
    odberatelcena, null, null, null, null, null, null, null, null, null, null);
  end if;
  
  select * into cenhlad from ccenhlad where idcenhlad = _idcenhlad;
  
  return calcceny(cenik.nakupCena, cenik.cenazakl,
    array[cenik.cena2, cenik.cena3, cenik.cena4, cenik.cena5], array[cenik.limmnoz2, cenik.limmnoz3, cenik.limmnoz4, cenik.limmnoz5],
    odberatelcena, cenhlad.idcenhlad, cenhlad.typcenyvychozik, cenhlad.typvypcenyk, cenhlad.typcenyvychozi25k, cenhlad.typvypceny25k, 
    cenhlad.zaokrjakk, cenhlad.zaokrnak, cenhlad.proczakl, 
    array[cenhlad.procento2, cenhlad.procento3, cenhlad.procento4, cenhlad.procento5], array[cenhlad.limmnoz2, cenhlad.limmnoz3, cenhlad.limmnoz4, cenhlad.limmnoz5]);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.calcceny(integer, integer, integer, integer) OWNER TO %firmaRole%;

-- vypocet cen pro hladinu/cenik, kde se vsechny potrebne udaje predavaji parametry
CREATE OR REPLACE FUNCTION calcceny(c_nakupCena numeric, c_cenazakl numeric,
  c_ceny numeric[], c_limmnoz numeric[], odberCena numeric,
  h_idcenhlad integer, h_typcenyvychozik varchar, h_typvypcenyk varchar, h_typcenyvychozi25k varchar, h_typvypceny25k varchar, 
  h_zaokrjakk varchar, h_zaokrnak varchar, h_proczakl numeric, h_procento numeric[], h_limmnoz numeric[])
  RETURNS numeric[] AS $$
declare
  vychozicena numeric;
  cena1 numeric;
  ceny numeric[];
  i integer;
begin
  -- otestujeme zda nemame cenu pro konretniho odberatele
  if odberCena is not null and odberCena > 0 then
    return array[odbercena, 0, 0, 0, 0, 0, 0, 0, 0, null];
  end if;

  if (h_idcenhlad is null) then
    for i in 1..4 loop
      ceny[i] = 0;
      if (c_limmnoz[i] > 0) then
        ceny[i] = c_ceny[i];
      end if;
    end loop;
    
    return c_cenazakl || ceny || c_limmnoz || cast(h_idcenhlad as numeric);
  end if;
  
  vychozicena = (CASE h_typcenyvychozik
    WHEN 'typCenyVychozi.nakCena' THEN c_nakupCena
    WHEN 'typCenyVychozi.zaklCena' THEN c_cenazakl
    ELSE null
  END);
  
  cena1 = calccena(vychozicena, h_typvypcenyk, h_proczakl);
  
  vychozicena = (CASE h_typcenyvychozi25k
    WHEN 'typCenyVychozi.nakCena' THEN c_nakupCena
    WHEN 'typCenyVychozi.zaklCena' THEN cena1
    ELSE null
  END);
  
  cena1 = zaokrouhli(cena1, h_zaokrjakk, h_zaokrnak);
  for i in 1..4 loop
    ceny[i] = 0;
    if (h_limmnoz[i] > 0) then
      ceny[i] = zaokrouhli(calccena(vychozicena, h_typvypceny25k, h_procento[i]), h_zaokrjakk, h_zaokrnak);
    end if;
  end loop;
  
  return cena1 || ceny || h_limmnoz || cast(h_idcenhlad as numeric);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.calcceny(numeric, numeric, numeric[], numeric[], numeric, integer, 
   varchar, varchar, varchar, varchar, varchar, varchar, numeric, numeric[], numeric[]) OWNER TO %firmaRole%;


-- uprava dat - je pred pohledem protoze v pohledu v DB jsou z neznamych duvodu vyjmenovane sloupce a nejde pak smazat
-- zkopirujeme skupiny firem do cenikovych skupin
insert into cskupcen(idskupcen, lastupdate, kod, nazev, nazeva, nazevb, nazevc, popis, poznam, "show", platido, platiod)
 (select idskupfir, now(), kod, nazev, nazeva, nazevb, nazevc, popis, poznam, "show", platido, platiod from askupfir);

-- nastavime sekvenci
select setval('seq_cskupcen', coalesce((select max(idskupcen) from cskupcen), 0) + 1);

-- nastavime na firmy
update aadresar set idskupcen = idskupfir where idskupfir is not null;
-- nastavime na odberatele
update codberatele set idskupcen = idskupfir where idskupfir is not null;
-- na cenove hladine prevedeme vsechnySkupFir na vsechnyFirmy
update ccenhlad set vsechnyfirmy = vsechnyskupfir;

-- na cenove hladiny misto skupin firem navazeme cenikove skupiny
insert into cskc_chl(idskupcen, idcenhlad)
 (select idskupfir, idcenhlad from cchl_skf);

-- odstranime vazebni tabulku hladin a skupin firem
DROP TABLE cchl_skf;
-- odstranime sloupec vsechnySkupFir
ALTER TABLE ccenhlad DROP COLUMN vsechnyskupfir;
-- odstranime skupinu firem z odberatelu
ALTER TABLE codberatele DROP COLUMN idskupfir;

-- pohled spojujici cenove hladiny s skupinami cen/firmami
CREATE OR REPLACE VIEW ccenhladusers AS

select cast(null as integer) as idskupcen, cast(null as integer) as idfirmy, H.*
from ccenhlad H
where H.vsechnyfirmy = true

union

select SC.idskupcen as idskupcen, cast(null as integer) as idfirmy, H.*
from ccenhlad H, cskc_chl SC
where H.vsechnyfirmy = false and H.idcenhlad = SC.idcenhlad

union

select null as idskupcen, F.idfirmy as idfirmy, H.*
from ccenhlad H, cchl_adr F
where H.vsechnyfirmy = false and H.idcenhlad = F.idcenhlad

union

-- hladina co neni nikde pouzita
select null as idskupcen, null as idfirmy, H.*
from ccenhlad H left outer join cskc_chl SC on H.idcenhlad = SC.idcenhlad 
left outer join cchl_adr F on H.idcenhlad = F.idcenhlad
where H.vsechnyfirmy = false and F.idfirmy is null and SC.idskupcen is null;

ALTER TABLE ccenhladusers OWNER TO %firmaRole%;

-- pohled spojujici predchozi pohled se skupinami zbozi
CREATE OR REPLACE VIEW ccenhladview AS

select cast(null as integer) as idskupzboz, H.*
from ccenhladusers H
where H.vsechnyskupzboz = true

union

select SZ.idskupzboz as idskupzboz, H.*
from ccenhladusers H, cchl_skz SZ
where H.vsechnyskupzboz = false and H.idcenhlad = SZ.idcenhlad;

ALTER TABLE ccenhladview OWNER TO %firmaRole%;


-- pridani agregacni funkce spojujici text
CREATE OR REPLACE FUNCTION public.sumtext_agg ( varchar, varchar )
RETURNS varchar AS $$
        SELECT (case when $1 is null then $2 else (case when $2 is null then $1 else $1 || '; ' || $2 end) end);
$$ LANGUAGE SQL IMMUTABLE STRICT;
ALTER FUNCTION sumtext_agg(character varying, character varying) OWNER TO %firmaRole%;
 
-- And then wrap an aggreagate around it
CREATE AGGREGATE public.sumtext (
        sfunc    = public.sumtext_agg,
        basetype = varchar,
        stype    = varchar
);

ALTER AGGREGATE sumtext(character varying) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 495, 't', %appVersion%, clock_timestamp());
