----------------------------------------------------------------------------------------------------------
-- Bug 24310 – [WinStrom] Specifikace možnosti platby a avíza o platbě
----------------------------------------------------------------------------------------------------------

CREATE SEQUENCE seq_dformauhradycis
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_dformauhradycis OWNER TO %firmaRole%;

CREATE TABLE dformauhradycis
(
  idformauhradycis integer NOT NULL DEFAULT nextval('seq_dformauhradycis'::regclass),
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  kod character varying(20) NOT NULL,
  nazev character varying(255) NOT NULL,
  nazeva character varying(255),
  nazevb character varying(255),
  nazevc character varying(255),
  popis text,
  poznam text,
  "show" boolean NOT NULL DEFAULT true,
  platido integer NOT NULL DEFAULT 9999,
  platiod integer NOT NULL DEFAULT 0,
  formauhrk character varying(50),
  kurz numeric(10,6) NOT NULL DEFAULT 0.0,
  kurzmnozstvi numeric(10,6) NOT NULL DEFAULT 0.0,
  limitvratky numeric(15,2) NOT NULL DEFAULT 0.0,
  idmeny integer,
  uuid character varying(50),
  CONSTRAINT dformauhradycis_pkey PRIMARY KEY (idformauhradycis),
  CONSTRAINT dformauhradycis_idmeny FOREIGN KEY (idmeny)
      REFERENCES umeny (idmeny) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE dformauhradycis OWNER TO %firmaRole%;

CREATE SEQUENCE seq_dformauhradyzauc
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_dformauhradyzauc OWNER TO %firmaRole%;

CREATE TABLE dformauhradyzauc
(
  idformauhradyzauc integer NOT NULL DEFAULT nextval('seq_dformauhradyzauc'::regclass),
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  idformauhradycis integer NOT NULL,
  idtypdokl integer NOT NULL,
  idkasa integer,
  CONSTRAINT dformauhradyzauc_pkey PRIMARY KEY (idformauhradyzauc),
  CONSTRAINT dformauhradyzauc_idtypdokl FOREIGN KEY (idtypdokl)
      REFERENCES dtypdokl (idtypdokl) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT dformauhradyzauc_idformauhradycis FOREIGN KEY (idformauhradycis)
      REFERENCES dformauhradycis (idformauhradycis) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT dformauhradyzauc_idkasa FOREIGN KEY (idkasa)
      REFERENCES dtypdokl (idtypdokl) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE dformauhradyzauc OWNER TO %firmaRole%;

CREATE SEQUENCE seq_dformauhrady
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_dformauhrady OWNER TO %firmaRole%;

CREATE TABLE dformauhrady
(
  idformauhrady integer NOT NULL DEFAULT nextval('seq_dformauhrady'::regclass),
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  castka numeric(15,2) NOT NULL DEFAULT 0.0,
  castkacenin numeric(15,2) NOT NULL DEFAULT 0.0,
  castkamen numeric(15,2) NOT NULL DEFAULT 0.0,
  cislokarty character varying(100),
  datum date,
  kurz numeric(10,6) NOT NULL DEFAULT 0.0,
  kurzmnozstvi numeric(10,6) NOT NULL DEFAULT 0.0,
  mnozcenin integer DEFAULT 0,
  strojparamuhr character varying(255),
  hotoveprijato numeric(15,2) NOT NULL DEFAULT 0.0,
  hotoveprijatomen numeric(15,2) NOT NULL DEFAULT 0.0,
  vratit numeric(15,2) NOT NULL DEFAULT 0.0,
  vratitmen numeric(15,2) NOT NULL DEFAULT 0.0,
  iddoklfak integer NOT NULL,
  idformauhradycis integer NOT NULL,
  idmeny integer,
  CONSTRAINT dformauhrady_pkey PRIMARY KEY (idformauhrady),
  CONSTRAINT dformauhrady_iddoklfak FOREIGN KEY (iddoklfak)
      REFERENCES ddoklfak (iddoklfak) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT dformauhrady_idformauhradycis FOREIGN KEY (idformauhradycis)
      REFERENCES dformauhradycis (idformauhradycis) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT dformauhrady_idmeny FOREIGN KEY (idmeny)
      REFERENCES umeny (idmeny) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE dformauhrady OWNER TO %firmaRole%;

-- indexy
CREATE INDEX dFormaUhrady_IdFormaUhradyCis_index ON dFormaUhrady USING btree (IdFormaUhradyCis);
CREATE INDEX dFormaUhrady_IdDoklFak_index ON dFormaUhrady USING btree (IdDoklFak);
CREATE INDEX dFormaUhrady_IdMeny_index ON dFormaUhrady USING btree (IdMeny);
CREATE INDEX dFormaUhradyCis_IdMeny_index ON dFormaUhradyCis USING btree (IdMeny);
CREATE INDEX dFormaUhradyZauc_IdFormaUhradyCis_index ON dFormaUhradyZauc USING btree (IdFormaUhradyCis);
CREATE INDEX dFormaUhradyZauc_IdTypDokl_index ON dFormaUhradyZauc USING btree (IdTypDokl);
CREATE INDEX dFormaUhradyZauc_IdKasa_index ON dFormaUhradyZauc USING btree (IdKasa);

-- pridani vazby na pohledavky
ALTER TABLE ddoklfak ADD COLUMN idformauhradyciskasa integer;
ALTER TABLE ddoklfak ADD CONSTRAINT ddoklfak_idformauhradyciskasa FOREIGN KEY (idformauhradyciskasa)
  REFERENCES dformauhradycis (idformauhradycis) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dDoklFak_IdFormaUhradyCisKasa_index ON dDoklFak USING btree (IdFormaUhradyCisKasa);

-- pridani vazby na pohledavky
CREATE OR REPLACE FUNCTION findvazbydokladu(_iddokl integer, _modul character varying) RETURNS SETOF vazbadokladu 
    AS $$
declare
  zaznam Record;
  _uroven integer;
  _prechodVP boolean;    -- zda povolen prechod z PPV/OBV na PPP/OBP
  _prechodZaucProd boolean;  -- zda jiz probehl prechod pres zauctovani prodejek
begin
  _uroven = 1;
  _prechodVP = true;
  _prechodZaucProd = false;
   
  drop table if exists vazbydokladu;
   
  CREATE TEMPORARY TABLE vazbydokladu
   (idvazebnidoklad integer NOT NULL,
    datucto timestamp without time zone,
    datvyst timestamp without time zone,
    iddokl integer,
    kod character varying(255),
    modul character varying(255),
    modulk character varying(255),
    popis text,
    poznam text,
    sumcelkem double precision,
    sumcelkemmen double precision,
    typvazbyk character varying(255),
    storno boolean,
    varsym character varying(255),
    idmeny integer,
    iduziv integer,
    idtypdokl integer,
    novy boolean,
    uroven integer,
    stavk character varying(255),
    nazfirmy character varying(255),
  	mesto character varying(255),
   	juhsum double precision,
    juhsummen double precision,
    CONSTRAINT vazbydokladu_pkey PRIMARY KEY (idvazebnidoklad)
  ) WITH (OIDS=FALSE);
   
  perform findViceVazeb('vazbydokladu', _idDokl, _modul, 0, true);
   
  while (select count(*) from vazbydokladu where novy = true) > 0 LOOP
    if _prechodZaucProd = false then
      select (count(*) > 0) into _prechodZaucProd from vazbydokladu where typvazbyk = 'typVazbyDokl.zaucProdejky';
    end if;
    if _prechodVP then
      select (count(*) = 0) into _prechodVP from vazbydokladu where modul in ('OBP', 'PPP') and novy = true;
    end if;
   
    for zaznam in (select * from vazbydokladu where novy = true)
    LOOP
      perform findViceVazeb('vazbydokladu', zaznam.idDokl, zaznam.modul, _uroven, _prechodVP);
   
      update vazbydokladu set novy = false where idvazebnidoklad = zaznam.idvazebnidoklad;
    end loop;

    if _prechodZaucProd then
      delete from vazbydokladu where novy = true and typvazbyk = 'typVazbyDokl.zaucProdejky';
    end if;
 
    _uroven = _uroven + 1;
  end loop;
   
  update vazbydokladu as V set modulK = modulK ||
    (case when (select typPohybuK from dDoklSklad where idDoklSklad = V.idDokl) = 'typPohybu.prijem' then 'PRI' else 'VYD' end)
    where V.modul = 'SKL';
 
  return query SELECT * FROM vazbydokladu;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.findvazbydokladu(_iddokl integer, _modul character varying) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION findvicevazeb(_inserttab character varying, _iddokl integer, _modul character varying, _uroven integer, _prechodvp boolean) RETURNS void
    AS $$ 
declare 
  tabulka varchar(20); 
  poleStav varchar(20); 
  dotaz varchar; 
  prechodPom varchar;   
begin 
  tabulka = getdoklnamepostfixbymodul(_modul); 
  prechodPom = ''; 
 
  if (_uroven = 0) then 
    if (tabulka = 'Fak') then 
      poleStav = 'stavUhrK'; 
    else 
      poleStav = 'stavUzivK';
      prechodPom = 'cast(null as numeric) as ';  
    end if; 
     
    dotaz =  
      'select 0 as idVazby, null as typVazbyK, false as storno, D.idDokl' || tabulka || ' as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.' || poleStav || ' as stavK,
        D.nazFirmy, D.mesto, ' || prechodPom || 'juhsum, ' || prechodPom || 'juhsummen
        from dDokl' || tabulka || ' as D where D.idDokl' || tabulka || ' = ' || _idDokl || ' and D.modul = ''' || _modul || ''''; 
  else 
 
  if (tabulka = 'Fak') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'', 
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'', ''typVazbyDokl.zaucProdejky'')  
          and V.modulB in (''BAN'', ''POK'', ''INT'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.hlavaSkl'' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.zaucProdejky'' and V.modulB in (''PHL'')
      union
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.zaucProdejky'' 
      union  
      select min(V.idVazby) as idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul 
        where V.idTabA in (select idPolFak from dPolFak where idDoklFak=' || _idDokl || ') and V.modulA=''' || _modul || ''' and 
          V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'')  
        group by V.typVazbyK, V.storno, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavuhrK, D.nazFirmy, D.mesto, D.juhsum, D.juhsummen 
      union  
      select min(V.idVazby) as idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on D.idDoklFak= 
                (select idDoklFak from dPolFak where idPolFak = V.idTabA) and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'') 
        group by V.typVazbyK, V.storno, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK, D.nazFirmy, D.mesto, D.juhsum, D.juhsummen 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')'; 
  else if (tabulka = 'Int') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'', 
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'', ''typVazbyDokl.zaucProdejky'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.hlavaSkl'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')  
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')'; 
  else if (tabulka = 'Sklad') then 
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and  
          V.modulA in (''FAV'',''FAP'',''PHL'',''ZAV'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl''  
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and  
          V.modulA in (''BAN'',''POK'',''INT'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl'' 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabA=D.idDoklSklad and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and
          V.modulA in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and
          V.modulB in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, 
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabA=D.idDoklSklad and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and 
          V.modulA in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklPrevod'' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, 
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and 
          V.modulB in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklPrevod'' 
      union   
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla'''; 
  else if (tabulka = 'Obch') then 
    if (_prechodVP = false and _modul in ('PPV', 'OBV')) then 
      prechodPom = ' and D.modul in (''PPV'', ''NAP'', ''OBV'')'; 
    end if; 
   
    dotaz =  
      'select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul  
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || ' 
      union  
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklObch as D on V.idTabB=D.idDoklObch and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || ' 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK,
        D.nazFirmy, D.mesto, D.juhsum, D.juhsummen
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'') 
      union 
      select V.idVazby, V.typVazbyK, V.storno, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,  
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK,
        D.nazFirmy, D.mesto, cast(null as numeric) as juhsum, cast(null as numeric) as juhsummen
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul  
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla'''; 
  else 
    raise exception 'findVazbyDokladu(): Neznámá tabulka % pro modul %', tabulka, _modul;  
  end if; 
  end if; 
  end if; 
  end if; 
   
  end if; 
   
  execute 'insert into ' || _insertTab || ' (idvazebnidoklad, typVazbyK, storno, idDokl, modul, modulk, kod, varsym, datvyst, datucto, sumcelkem, 
  sumcelkemmen, popis, poznam, idtypdokl,idmeny, iduziv, stavk, nazFirmy, mesto, juhsum, juhsummen, uroven, novy) select *, ' || _uroven || ', true from ('  
     || dotaz || ') as UN where (select count(*) from ' || _insertTab || ' as TV where  
     (TV.idDokl = UN.idDokl and TV.modul = UN.modul) or UN.idVazby = TV.idvazebnidoklad) = 0'; 
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION findvicevazeb(character varying, integer, character varying, integer, boolean) OWNER TO %firmaRole%;

-- vychozi formy uhrady
INSERT INTO dformauhradycis (lastupdate, kod, nazev, show, platiod, platido, formauhrk, kurz, kurzmnozstvi, limitvratky, uuid)
 VALUES ('2011-08-03 00:00:00.000', 'HOTOVE', 'Hotově', true, 0, 9999, 'formaUhr.hotove', 0.0, 0.0, 0.0, 'HOTOVE');
INSERT INTO dformauhradycis (lastupdate, kod, nazev, show, platiod, platido, formauhrk, kurz, kurzmnozstvi, limitvratky, uuid)
 VALUES ('2011-08-03 00:00:00.000', 'KARTA', 'Kartou', true, 0, 9999, 'formaUhr.platKart', 0.0, 0.0, 0.0, 'KARTA');

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 501, 't', %appVersion%, clock_timestamp());
