-- Bug 11396 – [WinStrom] Funkce Rezervovat
-- funkce pro zjisteni stavu na sklade zohlednujici rezervace pro danou firmu
CREATE OR REPLACE FUNCTION dgetstavnasklade(i_skladkartaid integer, i_datum date, i_polskladid integer, i_firmaid integer)
  RETURNS numeric AS
$BODY$
declare
  _sklad integer;
  _cenik integer;
  _stavMj numeric(19,6);
  _firmaRezerMj numeric(19,6);
  _celkemRezerMj numeric(19,6);
begin
  SELECT dgetstavnasklade(i_skladkartaid, i_datum, i_polskladid) INTO _stavMj;

  IF (i_firmaid > 0) THEN
    -- vymazeme jiz neplatne rezervace
    delete from srezervace where datumdo is not null and datumdo < date(now());
    -- nacteme si info ze skladove karty
    SELECT idCenik, idTypDokl INTO _cenik, _sklad FROM sKarty WHERE idKarty = i_skladkartaid;
    -- zjistime kolik ma firma rezervovano
    select coalesce(sum(mnozstvi), 0) into _firmaRezerMj from srezervace where idfirmy = i_firmaid and idcenik = _cenik and idsklad = _sklad;
    -- zjistime kolik je celkem rezervovano
    select coalesce(sum(mnozstvi), 0) into _celkemRezerMj from srezervace where idcenik = _cenik and idsklad = _sklad;
    
    -- spocteme kolik mu muzeme dat
    _stavMj := minValue(_stavMj, maxValue(_stavMj - _celkemRezerMj, 0) + _firmaRezerMj);
  END IF;
  
  RETURN _stavMj;  
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION dgetstavnasklade(integer, date, integer, integer) OWNER TO %firmaRole%;


-- Funkce odrezervujici dane mnozstvi z pro danou cenikou polozku na sklade
-- uprava aby zpracovaval rezervace postupne podle ID
CREATE OR REPLACE FUNCTION odrezervuj(_idFirmy integer, _idCenik integer, _idSkladu integer, vyskladni numeric(19,6))
  RETURNS void AS $$
declare
  _idRezer integer;
  _mnozstvi numeric(19,6);
  _vyskl numeric(19,6);
begin
  _vyskl = vyskladni;

  for _idRezer, _mnozstvi in select idRezervace, mnozstvi from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu order by idrezervace loop
    if _vyskl < _mnozstvi then
      update sRezervace set mnozstvi = (mnozstvi-_vyskl) where idRezervace = _idRezer;
      
      return;
    else
      delete from sRezervace where idRezervace = _idRezer;
      
      _vyskl = _vyskl - _mnozstvi;
    end if;
   
  end loop;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.odrezervuj(integer, integer, integer, numeric) OWNER TO %firmaRole%;


insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 66, 't');
