--------------------------------------------------------------------------------
-- Bug 33575 – [winstrom] tagy <_unknown_ /> v changes API
-- * trigger logchanges nefiltruje tabulky (wextid vynechána na úrovni triggerů)
-- * úplné vypuštění dalších tabulek je na pozdější diskusi, zatím zůstávají (lze ovlivnit v generování VO)
--------------------------------------------------------------------------------

DROP TRIGGER IF EXISTS wextidchanges ON wextid;

CREATE OR REPLACE FUNCTION logchanges() RETURNS trigger AS $$
    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    set modul null
    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
        if {[info exists NEW(modul)]} {
            set modul '[ quote $NEW(modul) ]'
        }
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
        if {[info exists OLD(modul)]} {
            set modul '[ quote $OLD(modul) ]'
        }
    }

    spi_exec "SELECT session_user AS user_name"
    set user_dollar_idx [string first \$ $user_name] 
    if {$user_dollar_idx != -1} {
        incr user_dollar_idx
        set user_name "[string range $user_name $user_dollar_idx end]" 
    }

    set operation "[string range $TG_op 0 0]"
    if {$TG_op == "INSERT"} {
        spi_exec "INSERT INTO changelog(tabulka, modul, id, operace, uzivatel)
            VALUES ('[ quote $TG_table_name ]',
                $modul,
                '[ quote $pkval ]',
                '[ quote $operation ]',
                '[ quote $user_name ]')"
    } elseif {$TG_op == "UPDATE" || $TG_op == "DELETE"} {
        set extid null

        if {[info exists OLD(kod)]} {
            append all_extids 'code:[ quote $OLD(kod) ]'
        }

        spi_exec "select attname AS fk from pg_attribute where attrelid = 'wextid'::regclass and attnum in (select conkey\[1\] from pg_constraint where conrelid = 'wextid'::regclass and confrelid = '[ quote $TG_table_name ]'::regclass)" {
            spi_exec "select extid from wextid where $fk = $pkval" {
                if {[info exists all_extids] && [string length $all_extids] > 0} {
                  append all_extids ","
                }
                append all_extids '[ quote $extid ]'
            }
        }

        if {[info exists all_extids] && [string length $all_extids] > 0} {
            set extid "array\[$all_extids\]"
        }

        spi_exec "INSERT INTO changelog(tabulka, modul, id, operace, uzivatel, extids)
            VALUES ('[ quote $TG_table_name ]',
                $modul,
                '[ quote $pkval ]',
                '[ quote $operation ]',
                '[ quote $user_name ]',
                $extid)"
    }

    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;

ALTER FUNCTION logchanges() OWNER TO %admin%;


-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  VALUES (nextval('seq_dbversion'), 723, 't', %appVersion%, clock_timestamp());
