--------------------------------------------------------------------------------
-- Bug 35401 – [FlexiBee] Přidat podporu pro zástupnosti workflow
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_wzastupnosti
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_wzastupnosti OWNER TO %firmaRole%;


CREATE TABLE wzastupnosti (
    idzastupnost integer DEFAULT nextval('seq_wzastupnosti'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
    datumod date,
    datumdo date,
    schvaleno boolean DEFAULT false NOT NULL,
    datumschvaleni timestamp without time zone,
    vyuzito boolean DEFAULT false NOT NULL,
    datumvyuziti timestamp without time zone,
    zablokovano boolean DEFAULT false NOT NULL,
    datumzablokovani timestamp without time zone,
    iduzivatelkdo integer NOT NULL,
    iduzivatelkoho integer NOT NULL,

    CONSTRAINT wzastupnosti_pkey PRIMARY KEY (idzastupnost),
    CONSTRAINT wzastupnosti_iduzivatelkdo FOREIGN KEY (iduzivatelkdo)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT wzastupnosti_iduzivatelkoho FOREIGN KEY (iduzivatelkoho)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE public.wzastupnosti OWNER TO %firmaRole%;

CREATE INDEX wzastupnosti_iduzivatelkdo_index ON wzastupnosti USING btree (iduzivatelkdo);
CREATE INDEX wzastupnosti_iduzivatelkoho_index ON wzastupnosti USING btree (iduzivatelkoho);



--------------------------------------------------------------------------------
-- Bug 35609 – [WinStrom] Fixopie pri vyrobe
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION dgetstavnaskladeaktualni(i_skladkartaid integer)
  RETURNS numeric AS
$BODY$       
declare      
  _idcenik integer;    
  _idskladu integer;    
  _iducetniobdobi integer;  
  _stavMj numeric(19,6);      
  _datum date;
begin      
  select idcenik, idskladu, iducetobdobi into _idcenik, _idskladu, _iducetniobdobi from skarty where idkarty = i_skladkartaid;  
  select platiDoData into _datum from nUcetObdobi where iducetobdobi = _iducetniobdobi;
  select dgetstavnaskladeaktualni(_idcenik, _idskladu, _datum) into _stavMj;
  RETURN _stavMj;       
end       
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dGetStavNaSkladeAktualni(integer) OWNER TO %firmaRole%;



--------------------------------------------------------------------------------
-- Bug 34389 – [FlexiBee] Batch API - dávková administrace FB (iT Highway)
--
-- FIX: rozšíření wuzivatel o "systemuser" (propagace příznaku z csuzivatel)
--------------------------------------------------------------------------------

ALTER TABLE wuzivatel ADD COLUMN systemuser boolean NOT NULL DEFAULT false;

UPDATE wuzivatel SET systemuser = true WHERE kod = 'system_user' AND NOT systemuser;



--------------------------------------------------------------------------------
-- Bug 35546 – [FlexiBee] - Majetek - doplnit účet "Zůstatková cena vyřazeného majetku"
--------------------------------------------------------------------------------

ALTER TABLE mmajetek ADD COLUMN idzustvyrazucet integer;
ALTER TABLE mtypmaj ADD COLUMN idzustvyrazucet integer;

ALTER TABLE mmajetek ADD CONSTRAINT mmajetek_idzustvyrazucet FOREIGN KEY (idzustvyrazucet)
  REFERENCES uucty (iducet) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE mtypmaj ADD CONSTRAINT mtypmaj_idzustvyrazucet FOREIGN KEY (idzustvyrazucet)
  REFERENCES uucty (iducet) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX mMajetek_IdZustVyrazUcet_index ON mMajetek USING btree (IdZustVyrazUcet);
CREATE INDEX mTypMaj_IdZustVyrazUcet_index ON mTypMaj USING btree (IdZustVyrazUcet);



-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  VALUES (nextval('seq_dbversion'), 766, 't', %appVersion%, clock_timestamp());
