------------------------------------------------------------------------------------------------
-- Bug 43086 - Špatně vypočtené odpisy, pokud nejsou účetní shodné s daňovými
------------------------------------------------------------------------------------------------

-- přidání nového sloupečku
alter table mudalosti add column sumdphdanove numeric(15,2) NOT NULL DEFAULT 0.0;


-- aktualizace funkcí triggerů
CREATE OR REPLACE FUNCTION mudalostiinsertucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
        -1, NEW.idStred, NEW.sumZkl, 0.0);

  -- u udalosti castecneho nebo uplneho vyrazeni musime zauctovat take zustatek udalosti
  if (NEW.typudalostik in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost (známe-li ji, tak použijeme tu)
    if (NEW.sumdph <> 0.0) then
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
            -1, NEW.idStred, NEW.sumdph, 0.0);
    else
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
            -1, NEW.idStred, NEW.sumdphdanove, 0.0);
    end if;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostiinsertucto() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION mudalostiupdateucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
       -1, OLD.idStred, -OLD.sumZkl, 0.0);

  -- pokud byla drive udalost castecne nebo uplne vyrazeni musime ji odebrat z obratu
  if (OLD.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost (známe-li ji, tak použijeme tu)
    if (OLD.sumdph <> 0.0) then
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
            -1, OLD.idStred, -OLD.sumdph, 0.0);
    else 
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
            -1, OLD.idStred, -OLD.sumdphdanove, 0.0);
    end if;
  end if;

  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme null a za sumZklMen 0.0
  execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
       -1, NEW.idStred, NEW.sumZKl, 0.0);

  -- pokud je udalost castecne nebo uplne vyrazeni musime zauctovat jeji zustatkovou cenu
  if (NEW.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost
    if (NEW.sumdph <> 0.0) then
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
            -1, NEW.idStred, NEW.sumdph, 0.0);
    else
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,
            -1, NEW.idStred, NEW.sumdphdanove, 0.0);
    end if;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostiupdateucto() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION mudalostideleteucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
       -1, OLD.idStred, -OLD.sumZkl, 0.0);

  -- pokud byla drive udalost castecne nebo uplne vyrazeni musime ji odebrat z obratu
  if (OLD.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost
    if (OLD.sumdph <> 0.0) then
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
            -1, OLD.idStred, -OLD.sumdph, 0.0);
    else
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,
            -1, OLD.idStred, -OLD.sumdph, 0.0);
    end if;
  end if;

  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostideleteucto() OWNER TO %firmaRole%;

-- vypneme triggery, aby nám neotevřeli zavřená účetní období
alter table mudalosti disable trigger mudalostiinsertucto; 
alter table mudalosti disable trigger mudalostiupdateucto; 
alter table mudalosti disable trigger mudalostideleteucto; 
-- nastavení hodnoty pro nové pole
update mudalosti set sumdphdanove = sumdph where sumdph <> 0.0;

-- u událostí, které jsou na majetku, který není účetně odepisován nastavíme sumdph na 0.0
update mudalosti set sumdph = 0.0 where sumdph <> 0.0 and idmajetku in (select idmajetku from mmajetek where nahrUcetOdpK = 'nahrUcet.ne');

alter table mudalosti enable trigger mudalostiinsertucto; 
alter table mudalosti enable trigger mudalostiupdateucto; 
alter table mudalosti enable trigger mudalostideleteucto; 

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
 values (nextval('seq_dbversion'), 941, 't', %appVersion%, clock_timestamp());
