<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:java="http://xml.apache.org/xalan/java" exclude-result-prefixes="java" >

  <xsl:output method="xml" indent="yes" encoding="utf-8" />
  <xsl:template match="winstrom">
    <winstrom version="1.0">
      <xsl:apply-templates select="*"/>
    </winstrom>
  </xsl:template>


    <!--
     DATUM,D
     TEXT,C,35
     MD_UCET,C,6
     MD_STR,N,3,0
     D_UCET,C,6
     D_STR,N,3,0
     KCS,N,12,2
     C_FAKTURY,C,16
     DAT_SPL,C,15
     OBDOBI,N,6,0
     DOKLAD,C,10
     KOD,C,1
     MENA,C,3
     HODNOTA,N,15,2
     KURZ,N,11,6
     OPRAVA,C,1
     UZAVRENO,C,1
     SPLATNOSTD,D
     PLACEN_DO,C,1
     STATUS_DO,C,1
     REZERVA1,C,10
     REZERVA2,C,7
     UPOM_POR,N,1,0
     KDOPORI,C,5
     KDYPORI,D
     OBD_DPH,N,6,0
     OZN_DPH,C,1
    -->

  <xsl:template match="*">
    <interni-doklad>
      <!-- id a kod vybirame podle typu dokladu-->
      <xsl:choose>
          <!-- z uctu vytvarime samostatne doklady, je M a ma vyplneny oba ucty = samostatny doklad -->
          <xsl:when test="substring(DOKLAD, 1, 4) = 'BEUR' or substring(DOKLAD, 1, 4) = 'BUSD' or substring(DOKLAD, 1, 4) = 'CSOB' or (substring(DOKLAD, 1, 1) = 'M' and string-length(D_UCET) != 0 and string-length(MD_UCET) != 0) or substring(DOKLAD, 1, 2) = 'op'">
              <id>ext:INNER:<xsl:value-of select="substring(translate(concat(DOKLAD, KCS), ',', ''), 1, 20)"/></id>
              <kod><xsl:value-of select="java:flexibee.Tool.toUpper(substring(translate(concat(DOKLAD, KCS), ',', ''), 1, 20))"/></kod>
          </xsl:when>

          <!-- je M a ma vyplnen jenom jeden ucet - id a kod z dokladu a prvnich 2 pismen textu -->
          <xsl:when test="substring(DOKLAD, 1, 1) = 'M'">
              <id>ext:INNER:<xsl:value-of select="concat(DOKLAD, substring(TEXT, 1, 2))"/></id>
              <kod><xsl:value-of select="concat(DOKLAD, substring(TEXT, 1, 2))"/></kod>
          </xsl:when>

          <xsl:when test="substring(DOKLAD, 1, 3) = 'zfd'">
              <id>ext:INNER:<xsl:value-of select="concat(DOKLAD, MENA)"/></id>
              <kod><xsl:value-of select="java:flexibee.Tool.toUpper(concat(DOKLAD, MENA))"/></kod>
          </xsl:when>

          <xsl:otherwise>
              <id>ext:INNER:<xsl:value-of select="DOKLAD"/></id>
              <!-- kod je unikatni, proto pokud menime jeho velikost pridame prefix $ -->
              <xsl:if test="java:flexibee.Tool.toUpper(DOKLAD) != DOKLAD">
                  <kod><xsl:value-of select="concat('$', java:flexibee.Tool.toUpper(DOKLAD))"/></kod>
              </xsl:if>
              <xsl:if test="java:flexibee.Tool.toUpper(DOKLAD) = DOKLAD">
                  <kod><xsl:value-of select="DOKLAD"/></kod>
              </xsl:if>
          </xsl:otherwise>
      </xsl:choose>
      
      <typDokl>code:INT. DOKLAD</typDokl>
      <datVyst><xsl:value-of select="java:flexibee.Tool.parseDate(DATUM, 'd.M.y')"/></datVyst>
      <datSplat><xsl:value-of select="java:flexibee.Tool.parseDate(SPLATNOSTD, 'd.M.y')"/></datSplat>
      <popis><xsl:value-of select="C_FAKTURY"/></popis>
      <xsl:choose>
        <xsl:when test="MENA != '' and MENA != '0'">
          <mena>code:<xsl:value-of select="MENA"/></mena>
		  <xsl:if test="KURZ != '0'">
          	<kurz><xsl:value-of select="java:flexibee.Tool.parseMoney(KURZ)"/></kurz>
		  </xsl:if>
        </xsl:when>
      </xsl:choose>
	  <xsl:if test="KDOPORI and KDOPORI != 'Blank' and KDOPORI != 'blank'">
      <uzivatel if-not-found="null">code:<xsl:value-of select="KDOPORI"/></uzivatel>
  	  </xsl:if>


      <!-- zkopirovani uctu -->
      <xsl:if test="(substring(DOKLAD, 1, 2) = 'FP' or substring(DOKLAD, 1, 1) = 'K' or substring(DOKLAD, 1, 1) = 'P' or substring(DOKLAD, 1, 1) = 'M' or substring(DOKLAD, 1, 2) = 'ZF' or substring(DOKLAD, 1, 3) = 'jsd' or substring(DOKLAD, 1, 3) = 'zfd') and D_UCET != ''">
          <protiUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(D_UCET/text())"/></protiUcet>
          <dphZaklUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(D_UCET/text())"/></dphZaklUcet>
      </xsl:if>

      <xsl:if test="(substring(DOKLAD, 1, 2) = 'FV' or substring(DOKLAD, 1, 1) = 'E') and MD_UCET != ''">
          <primUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(MD_UCET/text())"/></primUcet>
          <dphSnizUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(MD_UCET/text())"/></dphSnizUcet>
      </xsl:if>

      <!-- vytvoreni polozek -->
      <xsl:choose>
          <!-- polozku s uctem dal nevytvarime, jenom pokud maji vyplnen i md ucet - to je v pripade, ze mame jenom 1 polozku na dokladu
            u prijatych faktur, K, P, ID, M, ZF, jsd, zfd, op -->
          <xsl:when test="(substring(DOKLAD, 1, 2) = 'FP' or substring(DOKLAD, 1, 1) = 'K' or substring(DOKLAD, 1, 1) = 'P' or substring(DOKLAD, 1, 1) = 'M' or substring(DOKLAD, 1, 2) = 'ZF' or substring(DOKLAD, 1, 3) = 'jsd' or substring(DOKLAD, 1, 3) = 'zfd') and D_UCET != '' and string-length(MD_UCET) = 0">
          </xsl:when>

          <!-- polozku s uctem md nevytvarime, jenom pokud maji vyplnen i dal ucet - to je v pripade, ze mame jenom 1 polozku na dokladu
            u vydanych faktur -->
          <xsl:when test="(substring(DOKLAD, 1, 2) = 'FV' or substring(DOKLAD, 1, 1) = 'E') and MD_UCET != '' and string-length(D_UCET) = 0">
          </xsl:when>

          <!-- vytvarime polozku -->
          <xsl:otherwise>
              <polozkyIntDokladu>
                  <pokladni-pohyb-polozka>
                      <nazev><xsl:value-of select="TEXT"/></nazev>
                      <!-- podle pouziteho uctu rozhodneme o tom zda se jedna o dph nebo o zaklad -->
                      <xsl:choose>
                          <!-- jedna se o dph -->
                          <xsl:when test="substring(MD_UCET, 1, 3) = '343' or substring(D_UCET, 1, 3) = '343'">
                              <xsl:if test="MD_UCET != ''">
                                  <dphMdUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(MD_UCET/text())"/></dphMdUcet>
                                  <kopDphMdUcet>false</kopDphMdUcet>
                              </xsl:if>

                              <xsl:if test="D_UCET != ''">
                                  <dphDalUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(D_UCET/text())"/></dphDalUcet>
                                  <kopDphDalUcet>false</kopDphDalUcet>
                              </xsl:if>

                              <!-- polozky jednostranneho uctovani -->
                              <xsl:if test="(substring(DOKLAD, 1, 4) = 'CSOB' or substring(DOKLAD, 1, 4) = 'BEUR' or substring(DOKLAD, 1, 4) = 'BUSD' or substring(DOKLAD, 1, 3) = 'dpz' or substring(DOKLAD, 1, 1) = 'f' or substring(DOKLAD, 1, 1) = 'b' or substring(DOKLAD, 1, 1) = 'i' or substring(DOKLAD, 1, 1) = 'p' or substring(DOKLAD, 1, 2) = 'DV' or substring(DOKLAD, 1, 2) = 'BV' or substring(DOKLAD, 1, 3) = 'DPH') and string-length(MD_UCET) = 0">
                                  <dphMdUcet>code:395999</dphMdUcet>
                                  <kopDphMdUcet>false</kopDphMdUcet>
                              </xsl:if>

                              <xsl:if test="(substring(DOKLAD, 1, 4) = 'CSOB' or substring(DOKLAD, 1, 4) = 'BEUR' or substring(DOKLAD, 1, 4) = 'BUSD' or substring(DOKLAD, 1, 3) = 'dpz' or substring(DOKLAD, 1, 1) = 'f' or substring(DOKLAD, 1, 1) = 'b' or substring(DOKLAD, 1, 1) = 'i' or substring(DOKLAD, 1, 1) = 'p' or substring(DOKLAD, 1, 2) = 'DV' or substring(DOKLAD, 1, 2) = 'BV' or substring(DOKLAD, 1, 3) = 'DPH') and string-length(D_UCET) = 0">
                                  <dphDalUcet>code:395999</dphDalUcet>
                                  <kopDphDalUcet>false</kopDphDalUcet>
                              </xsl:if>
                              
                              <sumDph><xsl:value-of select="java:flexibee.Tool.parseMoney(KCS)"/></sumDph>
                          </xsl:when>

                          <!-- jedna se o zaklad -->
                          <xsl:otherwise>
                              <xsl:if test="MD_UCET != ''">
                                  <zklMdUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(MD_UCET/text())"/></zklMdUcet>
                                  <kopZklMdUcet>false</kopZklMdUcet>
                              </xsl:if>

                              <xsl:if test="D_UCET != ''">
                                  <zklDalUcet>code:<xsl:value-of select="java:flexibee.Tool.correctUcet(D_UCET/text())"/></zklDalUcet>
                                  <kopZklDalUcet>false</kopZklDalUcet>
                              </xsl:if>

                              <!-- polozky jednostranneho uctovani -->
                              <xsl:if test="(substring(DOKLAD, 1, 4) = 'CSOB' or substring(DOKLAD, 1, 4) = 'BEUR' or substring(DOKLAD, 1, 4) = 'BUSD' or substring(DOKLAD, 1, 3) = 'dpz' or substring(DOKLAD, 1, 1) = 'f' or substring(DOKLAD, 1, 1) = 'b' or substring(DOKLAD, 1, 1) = 'i' or substring(DOKLAD, 1, 1) = 'p' or substring(DOKLAD, 1, 2) = 'op' or substring(DOKLAD, 1, 2) = 'DV' or substring(DOKLAD, 1, 2) = 'BV' or substring(DOKLAD, 1, 3) = 'DPH') and string-length(MD_UCET) = 0">
                                  <zklMdUcet>code:395999</zklMdUcet>
                                  <kopZklMdUcet>false</kopZklMdUcet>
                              </xsl:if>

                              <xsl:if test="(substring(DOKLAD, 1, 4) = 'CSOB' or substring(DOKLAD, 1, 4) = 'BEUR' or substring(DOKLAD, 1, 4) = 'BUSD' or substring(DOKLAD, 1, 3) = 'dpz' or substring(DOKLAD, 1, 1) = 'f' or substring(DOKLAD, 1, 1) = 'b' or substring(DOKLAD, 1, 1) = 'i' or substring(DOKLAD, 1, 1) = 'p' or substring(DOKLAD, 1, 2) = 'op' or substring(DOKLAD, 1, 2) = 'DV' or substring(DOKLAD, 1, 2) = 'BV' or substring(DOKLAD, 1, 3) = 'DPH') and string-length(D_UCET) = 0">
                                  <zklDalUcet>code:395999</zklDalUcet>
                                  <kopZklDalUcet>false</kopZklDalUcet>
                              </xsl:if>
                              
                              <sumZkl><xsl:value-of select="java:flexibee.Tool.parseMoney(KCS)"/></sumZkl>
                              <typSzbDphK>typSzbDph.dphOsv</typSzbDphK>
                          </xsl:otherwise>
                      </xsl:choose>
                      
                      <typCenyDphK>typCeny.bezDph</typCenyDphK>
                  </pokladni-pohyb-polozka>
              </polozkyIntDokladu>
          </xsl:otherwise>
      </xsl:choose>
    </interni-doklad>
  </xsl:template>
</xsl:stylesheet>
